/* -*-C-*-
 *
 * $Revision: 1.1.1.1 $
 *   $Author: drusling $
 *     $Date: 1998/09/22 15:48:17 $
 *
 * Copyright (c) 1996 Advanced RISC Machines Limited.
 * All Rights Reserved.
 *
 *   Project: ANGEL
 *
 *     Title: Device tables -- just one serial device for now.
 */

#include "devdriv.h"

/*
 * Headers declaring the angel_DeviceEntry struct for each device
 * should be included here:
 */

#include "serial.h"             /* serial device */


#if (ETHERNET_SUPPORTED == 1)
#include "arm_ether.h"
#endif

/*
 * The master device table - one entry per device
 * ORDER AND NUMBER OF ENTRIES MUST MATCH enum DeviceIdent IN devices.h
 */
const struct angel_DeviceEntry * const angel_Device[DI_NUM_DEVICES] = {
  &angel_SerialDevice
#if (ETHERNET_SUPPORTED == 1)
        ,&angel_EthernetDevice
#endif
};


/*
 * The interrupt handler table - one entry per handler.
 *
 * DE_NUM_INT_HANDLERS must be set in devdriv.h to the number of
 * entries in this table.
 */

#if (DE_NUM_INT_HANDLERS>0)

const struct angel_IntHandlerEntry angel_IntHandler[DE_NUM_INT_HANDLERS] = {
  { angel_SerialIntHandler, SER_DEV_IDENT_0 }
};

#endif


/*
 * The poll handler table - one entry per handler 
 *
 * DE_NUM_POLL_HANDLERS must be set in devdriv.h to the number of
 * entries in this table. Nope, untrue. 
 */

#if (DE_NUM_POLL_HANDLERS>0)

        /* Need to add ethernet poll handler */
const struct angel_PollHandlerEntry angel_PollHandler[DE_NUM_POLL_HANDLERS] = {
    { angel_EthernetPoll, 0, angel_EthernetNOP, DI_ETHERNET }
};

#endif

/* EOF devices.c */
