;/***********************************************************************
;* Copyright  Intel Corporation 1998.  All rights reserved.
;************************************************************************
;
;  SA-110 MMU specifics - constants, registers etc.
;
;       $Id$
;
;************************************************************************/


 ; Allow different cache sizes - SA1100
 IF :LNOT: :DEF: DCACHE_SIZE
DCACHE_SIZE             EQU     0x4000  ; 16kB Dcache
 ENDIF

DCACHE_LINE             EQU     0x20    ; 32B cache line entry

L1_TABLE_ENTRIES        EQU     0x1000  ; 16kB table (word entries) 

 ; Allow different sized L2 Page tables, each L1 entry needs 256 words (1KB)
 IF :LNOT: :DEF: L2_TABLE_ENTRIES
L2_TABLE_ENTRIES        EQU     0x100   ;  1kB table (word entries) 
 ENDIF

L2_CONTROL              EQU     0x1     ; domain0, page table pointer

; Access Permissions
; Depending on the setup, 0 represents one of these two access permissions
PT_NO_ACCESS            EQU     (0 << 10)
PT_SVC_R                EQU     (0 << 10)
PT_SVC_RW               EQU     (1 << 10)
PT_NO_USR_W             EQU     (2 << 10)
PT_ALL_ACCESS           EQU     (3 << 10)

PT_C_BIT                EQU     (1 << 3)
PT_B_BIT                EQU     (1 << 2)
PT_CB_BITS              EQU     (PT_C_BIT + PT_B_BIT)


; Entry type
PT_SECTION              EQU     2
PT_PAGE                 EQU     1
PT_INVALID              EQU     0

PT_SMALL                EQU     2
PT_LARGE                EQU     1


;DRAM_ACCESS            EQU     0xC0E   ; AP=11, domain0, C=1, B=1
DRAM_ACCESS             EQU     (PT_ALL_ACCESS + PT_CB_BITS + PT_SECTION)
; Non-cached, buffered access
NCDRAM_ACCESS           EQU     (PT_ALL_ACCESS + PT_B_BIT + PT_SECTION)
;FLASH_ACCESS           EQU     0x80A   ; AP=10, domain0, C=1, B=0
FLASH_ACCESS            EQU     (PT_NO_USR_W + PT_C_BIT + PT_SECTION)
;IO_ACCESS              EQU     0xC02   ; AP=11, domain0, C=0, B=0
IO_ACCESS               EQU     (PT_ALL_ACCESS + PT_SECTION)
SSRAM_ACCESS            EQU     0x0FFD  ; AP=11, domain0, C=1, B=1
EPROM_ACCESS            EQU     0x0AA9  ; AP=10, domain0, C=1, B=0


; Definitions used in conditional assembly of Icache, Dcache and Write Buffer
; options

IC_ON           EQU     0x1000
IC_OFF          EQU     0x0

DC_ON           EQU     0x4
DC_OFF          EQU     0x0

WB_ON           EQU     0x8
WB_OFF          EQU     0x0


; Bit definitions for the control register: 

; enables are logically OR'd with the control register
; use bit clears (BIC's) to disable functions 
;       *** all bits cleared on RESET ***

EnableMMU               EQU     0x1
EnableAlignFault        EQU     0x2
EnableDcache            EQU     0x4
EnableWB                EQU     0x8
EnableBigEndian         EQU     0x80
EnableMMU_S             EQU     0x100           ; selects MMU access checks 
EnableMMU_R             EQU     0x200           ; selects MMU access checks 
EnableIcache            EQU     0x1000

        END
