/*
 * Copyright (C) 1996
 * Digital Equipment Corporation, Maynard, Massachusetts.
 * All Rights Reserved
 *
 * Permission to  use, copy,  modify, and distribute  this software
 * and its documentation for any  purpose and without fee is hereby 
 * granted, provided that the  copyright notice and this permission 
 * notice   appear  in  all   copies  of  software  and  supporting  
 * documentation,  and  that  the  name  of Digital  not be used in 
 * advertising or  publicity  pertaining to   distribution  of  the 
 * software  without  specific,  written prior  permission. Digital  
 * grants  this  permission  provided  that you  prominently  mark, 
 * as  not part  of the original,  any modifications that  is  made  
 * to this software or documentation.
 *
 * Digital Equipment Corporation  disclaims all  warranties  and/or 
 * guarantees with  regard to  this software, including all implied 
 * warranties   of  fitness  for   a   particular    purpose    and   
 * merchantibility, and makes no representations regarding  use of,  
 * or  the  results of  the use of, the  software and documentation 
 * in terms  of correctness, accuracy, reliability, currentness, or 
 * otherwise;  and   you rely  on  the software,  documentation and 
 * results solely at your own risk.
 *
 * ======================================================================
 *
 * Abstract:          PCI specific header file for StrongARM.
 *
 *	$Id: pci.h,v 1.1 1999/01/26 14:16:34 mquinn Exp $
 *
 */

#ifndef __pci_h		/* Only include stuff once */
#define __pci_h

#define PCI_CFG_TYPE_0	0xFB000000                                                                        
#define PCI_CFG_TYPE_1	0xFA000000

/* Offsets used to convert bus, device & function to config (and back) */
#define PCI_BUSOFFSET	16
#define PCI_DEVOFFSET	11
#define PCI_FUNCOFFSET	 8

/* Bit masks used to convert bus, device & function to config */
#define PCI_BUSMASK	0xFF
#define PCI_DEVMASK	0x1F
#define PCI_FUNCMASK	0x7

#define GET_BUS(x)      ((x >> PCI_BUSOFFSET) & PCI_BUSMASK) 
#define GET_DEVICE(x)   ((x >> PCI_DEVOFFSET) & PCI_DEVMASK) 
#define GET_FUNCTION(x) ((x >> PCI_FUNCOFFSET) & PCI_FUNCMASK) 

#define MAKE_HANDLE(b,x,y,z)	(U32)((U32)b | \
				(U32)((x & PCI_BUSMASK) << PCI_BUSOFFSET) | \
				(U32)((y & PCI_DEVMASK) << PCI_DEVOFFSET) | \
				(U32)((z & PCI_FUNCMASK) << PCI_FUNCOFFSET))

#define MAX_PCI_BUS     1
#define MAX_PCI_SLOT    11


#define PCI_VENDOR_ID	0x00
#define PCI_DEVICE_ID	0x02
#define PCI_COMMAND	0x04
#define PCI_STATUS	0x06
#define PCI_REV_ID	0x08
#define PCI_CLASS_ID	0x09
#define PCI_MEM_BAR	0x10
#define PCI_ROM_BAR	0x30
#define PCI_LATENCY	0x0D
#define PCI_MAX_BAR	6

/* From ebsa285/platform.[sh] (well, they are PCI specific): */

/*  PCI Command Register
 */
#define PCI_IO_ENABLE                   0x0001
#define PCI_MEM_ENABLE                  0x0002
#define PCI_MASTER_ENABLE               0x0004
#define PCI_MEM_WRINVALID               0x0010
#define PCI_FAST_B2B                    0x0200
/*  Another workaround the Assembler limitation. PCI_CFN_INIT should be:
 * PCI_CFN_INIT		EQU	(PCI_IO_ENABLE | PCI_MEM_ENABLE | PCI_MASTER_ENABLE | PCI_MEM_WRINVALID)
 */
#define PCI_CFN_ENABLE                  (PCI_IO_ENABLE | PCI_MEM_ENABLE)
#define PCI_MAS_ENABLE                  (PCI_MEM_WRINVALID | PCI_MASTER_ENABLE)
#define PCI_CFN_INIT                    (PCI_CFN_ENABLE | PCI_MAS_ENABLE)

/*  PCI Outbound Interrupt Status/Mask
 */
#define PCI_OUT_DOORBELL                0x04
#define PCI_OUT_POSTLIST                0x08
#define NO_OUTBOUND_INTS                (PCI_OUT_DOORBELL | PCI_OUT_POSTLIST)

#define PCI_INVALID			0xFFFFFFFF
#define PCI_NOTFITTED			0xFFFF

/* PCI Class Codes */
#define PCI_HISTORY_CLASS		0x00
#define PCI_STORAGE_CLASS		0x01
#define PCI_NETWORK_CLASS		0x02
#define PCI_DISPLAY_CLASS		0x03
#define PCI_MMEDIA_CLASS		0x04
#define PCI_MEMORY_CLASS		0x05
#define PCI_BRIDGE_CLASS		0x06
#define PCI_COMMS_CLASS			0x07
#define PCI_PERIPH_CLASS		0x08
#define PCI_INPUT_CLASS			0x09
#define PCI_DOCKING_CLASS		0x0A
#define PCI_PROCESS_CLASS		0x0B
#define PCI_SERIAL_CLASS		0x0C
#define PCI_MISC_CLASS			0xFF

/* PCI Sub-class codes */
#define PCI_OLDPCI_SUB			0x00
#define PCI_OLDVGA_SUB			0x01

#define PCI_SCSI_SUB			0x00
#define PCI_IDE_SUB			0x01
#define PCI_FLOPPY_SUB			0x02
#define PCI_IPI_SUB			0x03
#define PCI_RAID_SUB			0x04

#define PCI_ETHER_SUB			0x00
#define PCI_TOKEN_SUB			0x01
#define PCI_FDDI_SUB			0x02
#define PCI_ATM_SUB			0x03

#define PCI_VGA_SUB			0x00	/* v Different interface byte */
#define PCI_8541_SUB			0x00	/* ^ Different interface byte */
#define PCI_XGA_SUB			0x01

#define PCI_VIDEO_SUB			0x00
#define PCI_AUDIO_SUB			0x01

#define PCI_RAM_SUB			0x00
#define PCI_FLASH_SUB			0x01

#define PCI_HOST_SUB			0x00
#define PCI_2ISA_SUB			0x01
#define PCI_2EISA_SUB			0x02
#define PCI_2MCA_SUB			0x03
#define PCI_2PCI_SUB			0x04
#define PCI_2PCMCIA_SUB			0x05
#define PCI_2NUBUS_SUB			0x06
#define PCI_2CARDBUS_SUB		0x07

#define PCI_XTSER_SUB			0x00
#define PCI_16450_SUB			0x01	/* Interface is 0x01 */
#define PCI_16550_SUB			0x01	/* Interface is 0x02 */
#define PCI_PARALLEL_SUB		0x02	/* Interface is 0x00 */
#define PCI_LPT_SUB			0x02	/* Interface is 0x01 */
#define PCI_ECP_SUB			0x02	/* Interface is 0x02 */

#define PCI_8259_SUB			0x00	/* Interface is 0x00 */
#define PCI_PICISA_SUB			0x00	/* Interface is 0x01 */
#define PCI_PICEISA_SUB			0x00	/* Interface is 0x02 */
#define PCI_8237_SUB			0x01	/* Interface is 0x00 */
#define PCI_DMAISA_SUB			0x01	/* Interface is 0x01 */
#define PCI_DMAEISA_SUB			0x01	/* Interface is 0x02 */
#define PCI_8254_SUB			0x02	/* Interface is 0x00 */
#define PCI_TIMERISA_SUB		0x02	/* Interface is 0x01 */
#define PCI_TIMEREISA_SUB		0x02	/* Interface is 0x02 */
#define PCI_RTC_SUB			0x03	/* Interface is 0x00 */
#define PCI_RTCISA_SUB			0x03	/* Interface is 0x01 */

#define PCI_KEYBD_SUB			0x00
#define PCI_PEN_SUB			0x01
#define PCI_MOUSE_SUB			0x02

#define PCI_DOCK_SUB			0x00

#define PCI_386_SUB			0x00
#define PCI_486_SUB			0x01
#define PCI_PENTIUM_SUB			0x02
#define PCI_ALPHA_SUB			0x10
#define PCI_POWERPC_SUB			0x20
#define PCI_COPROC_SUB			0x40

#define PCI_1394_SUB			0x00
#define PCI_ACCESS_SUB			0x01
#define PCI_SSA_SUB			0x02
#define PCI_USB_SUB			0x03
#define PCI_FIBRE_SUB			0x04

/* A few 24-bit device classes.. */
#define PCI_ETHERNET_CLASS	(((PCI_STORAGE_CLASS<<8) + PCI_ETHER_SUB)<<8)
#define PCI_VGA_CLASS		(((PCI_DISPLAY_CLASS<<8) + PCI_VGA_SUB)<<8)
#define PCI_OLDVGA_CLASS	(((PCI_HISTORY_CLASS<<8) + PCI_OLDVGA_SUB)<<8)
#define PCI_SCSI_CLASS		(((PCI_STORAGE_CLASS<<8) + PCI_SCSI_SUB)<<8)
#define PCI_2PCI_CLASS		(((PCI_BRIDGE_CLASS<<8) + PCI_2PCI_SUB)<<8)
#define PCI_2ISA_CLASS		(((PCI_BRIDGE_CLASS<<8) + PCI_2ISA_SUB)<<8)

/* defined in pci.c */
#if uHAL_PCI != 0

extern U8	 SAr_PciCfgReadByte(U32 devHandle, U32 offset, PS32 pStatus) ;
extern U16	 SAr_PciCfgReadShort(U32 devHandle, U32 offset, PS32 pStatus) ;
extern U32	 SAr_PciCfgReadLong(U32 devHandle, U32 offset, PS32 pStatus) ;
extern void	 SAr_PciCfgWriteByte(U32 devHandle, U32 offset, U32 data, PS32 pStatus) ;
extern void	 SAr_PciCfgWriteShort(U32 devHandle, U32 offset, U32 data, PS32 pStatus) ;
extern void	 SAr_PciCfgWriteLong(U32 devHandle, U32 offset, U32 data, PS32 pStatus) ;
extern void	 SAr_ConfigurePciDevices(void) ;
extern U32	 SAr_GetNextMemBAR(U32 handle, U32 *bar) ;
extern U32	 SAr_GetNextIOBAR(U32 handle, U32 *bar) ;
extern U32	 SAr_FindPCIDevClass(U32 devClass, U32 *inst) ;
extern U32	 SAr_FindPCIDevice(U16 vendor, U16 device, U32 *inst) ;
extern void	*SAr_Mem2Pci(void *memAddress, U32 *status) ;

/* Low-level PCI setup routines */
extern void	 SAir_AssignResources(U32 handle, U32 *sizePtr);
extern U32	 SAir_GetNextPciAddr(U32 *base, U32 size) ;
extern U32	 SAir_AssignMemAddr(U32 size) ;
extern U32	 SAir_AssignIoAddr(U32 size) ;
extern U32	 SAir_ReadBARSize(U32 handle, U32 *sizePtr, U32 offset) ;
extern S32	 SAir_WriteBARSize(U32 handle, U32 data, U32 size, U32 offset) ;

/* Aliases for direct PCI access routines */
#define SAr_GetPciVendorID(h, s) SAr_PciCfgReadShort(h, PCI_VENDOR_ID, s)
#define SAr_GetPciDeviceID(h, s) SAr_PciCfgReadShort(h, PCI_DEVICE_ID, s)
/* Class ID is top 24 bits, Revision ID is bottom 8 bits */
#define SAr_GetPciClassID(h, s)	 (SAr_PciCfgReadLong(h, PCI_REV_ID, s) >> 8)
#define SAr_GetPciRevID(h, s)	 SAr_PciCfgReadByte(h, PCI_REV_ID, s)
#define SAr_GetPciBAR(h, o, s)	 SAr_PciCfgReadLong(h, PCI_MEM_BAR + (o * 4), s)
#define SAr_GetPciCommand(h, s)	 SAr_PciCfgReadShort(h, PCI_COMMAND, s)
#define SAr_GetPciStatus(h, s)	 SAr_PciCfgReadShort(h, PCI_STATUS, s)

#endif		/* uHAL_PCI */

#endif		/* __pci_h */

