;/***********************************************************************
;* Copyright  Intel Corporation 1998.  All rights reserved.
;************************************************************************
;
;  EBSA285 address map
;
;       NOTE: This is a multi-hosted header file for use with uHAL and
;             supported debuggers.
;
;       $Id: platform.s,v 1.31 1998/12/09 16:44:35 mquinn Exp $
;
;***********************************************************************/

 IF :LNOT: :DEF: __address_h
__address_h                     EQU     1

PLATFORM_ID                     EQU     0x00000008

; Common modules for uHAL can be included or excluded by changing these
; definitions. These can be over-ridden by the makefile/ARM project file
; provided the .h file can is rebuilt.
 IF :LNOT: :DEF: uHAL_BOOT
uHAL_BOOT                       EQU     1
 ENDIF
 IF :LNOT: :DEF: uHAL_HEAP
uHAL_HEAP_BASE                  EQU     (PCI_DRAMBASE - 0x300000)
uHAL_HEAP_SIZE                  EQU     0x200000        
uHAL_HEAP                       EQU     1
 ENDIF
 IF :LNOT: :DEF: uHAL_TIMERS
uHAL_TIMERS                     EQU     1
 ENDIF
 IF :LNOT: :DEF: uHAL_INTERRUPTS
uHAL_INTERRUPTS                 EQU     1
 ENDIF
 IF :LNOT: :DEF: uHAL_COMPLEX_IRQ
uHAL_COMPLEX_IRQ                EQU     0
 ENDIF
 IF :LNOT: :DEF: uHAL_PCI
uHAL_PCI                        EQU     1
 ENDIF
 IF :LNOT: :DEF: uHAL_VGA
uHAL_VGA                        EQU     0
 ENDIF
 IF :LNOT: :DEF: uHAL_LCD
uHAL_LCD                        EQU     0
 ENDIF

; /* handy sizes */
SZ_1K                           EQU     0x00000400
SZ_16K                          EQU     0x00004000
SZ_64K                          EQU     0x00010000
SZ_256K                         EQU     0x00040000
SZ_512K                         EQU     0x00080000

SZ_1M                           EQU     0x00100000
SZ_2M                           EQU     0x00200000
SZ_4M                           EQU     0x00400000
SZ_8M                           EQU     0x00800000
SZ_16M                          EQU     0x01000000
SZ_32M                          EQU     0x02000000
SZ_64M                          EQU     0x04000000
SZ_128M                         EQU     0x08000000
SZ_256M                         EQU     0x10000000

SZ_1G                           EQU     0x40000000

; /* DRAM space */
DRAM_BASE                       EQU     0x00000000
DRAM_DEFAULT                    EQU     SZ_16M
PCI_DRAMSIZE                    EQU     SZ_2M
PCI_DRAMBASE                    EQU     (DRAM_DEFAULT - PCI_DRAMSIZE)

; /* DC21285 Addresses */

DC21285_DRAM_A0MR               EQU     0x40000000
DC21285_DRAM_A1MR               EQU     0x40004000
DC21285_DRAM_A2MR               EQU     0x40008000
DC21285_DRAM_A3MR               EQU     0x4000C000
        
DC21285_XBUS_XCS0               EQU     0x40010000
DC21285_XBUS_XCS1               EQU     0x40011000
DC21285_XBUS_XCS2               EQU     0x40012000
DC21285_XBUS_NOCS               EQU     0x40014000

DC21285_FLASH                   EQU     0x41000000
; /* FLASH base */
FLASH_BASE                      EQU     DC21285_FLASH
FLASH_SIZE                      EQU     SZ_4M
FLASH_END                       EQU     (FLASH_BASE + FLASH_SIZE - 1)
; /* ROM is really FLASH on EBSA-285, but you can call it ROM if you want.. */
DC21285_ROM                     EQU     DC21285_FLASH
; /* ROM base */
EPROM_BASE                      EQU     DC21285_FLASH
EPROM_SIZE                      EQU     FLASH_SIZE
EPROM_END                       EQU     (EPROM_BASE + EPROM_SIZE - 1)
ROMTop                          EQU     EPROM_END


DC21285_ARMCSR_BASE             EQU     0x42000000

DC21285_SA_CACHE_FLUSH          EQU     0x50000000
; /* Reserve Virtual Address for cache flush */
CLEAN_BASE                      EQU     DC21285_SA_CACHE_FLUSH

DC21285_OUTBOUND_WRITE_FLUSH    EQU     0x78000000

DC21285_PCI_IACK                EQU     0x79000000
        
DC21285_PCI_TYPE_1_CONFIG       EQU     0x7A000000

DC21285_PCI_TYPE_0_CONFIG       EQU     0x7B000000

DC21285_PCI_IO                  EQU     0x7C000000

DC21285_PCI_MEMORY              EQU     0x80000000

PCI_IO                          EQU     DC21285_PCI_IO
PCI_MEM                         EQU     DC21285_PCI_MEMORY

; macro to standardise conversion from ISA (X bus) to sparse address..
; and some to standardise conversion from mapped memory to PCI
;#define _MapAddress(a)         (((unsigned int) a << 2) | DC21285_XBUS_XCS1)
;#define _MapIOAddress(a)       ((U32)PCI_IO + (U32)(a))
;#define _MapMemAddress(a)      ((U32)PCI_MEM + (U32)(a))
        

COM1Port        EQU     0x3f8
COM2Port        EQU     0x2f8           
IOShift         EQU     2

; /* Serial port constants */
XBUS_DEV_0      EQU     DC21285_XBUS_XCS0
XBUS_DEV_1      EQU     DC21285_XBUS_XCS1
XBUS_DEV_2      EQU     DC21285_XBUS_XCS2
XBUS_DEV_3      EQU     DC21285_XBUS_NOCS
; /* Should be debug only, use COM1Port not COM1 */
COM1            EQU     (XBUS_DEV_1 + (COM1Port << IOShift))
COM2            EQU     (XBUS_DEV_1 + (COM2Port << IOShift))

; /* LED definitions */
SA_LED_ON                       EQU     0
SA_LED_OFF                      EQU     1

LED_BANK                        EQU     XBUS_DEV_2
RED_LED         EQU     0x04
GREEN_LED       EQU     0x02
YELLOW_LED      EQU     0x01
ALL_LEDS        EQU     0x07
SA_NUM_OF_LEDS  EQU     3
;#define SA_LED_MASKS   { 0, YELLOW_LED, GREEN_LED, RED_LED }
;#define SA_LED_OFFSETS { 0, (void *)LED_BANK, (void *)LED_BANK, (void *)LED_BANK }
 IF :LNOT: :DEF: HIGH
HIGH            EQU     1
 ENDIF


CSR_BASE                EQU     DC21285_ARMCSR_BASE

PCI_COMMAND             EQU     0x04
PCI_MEM_BAR             EQU     0x10
PCI_IO_BAR              EQU     0x14
PCI_DRAM_BAR            EQU     0x18
PCI_OUT_INT_STATUS      EQU     0x30
PCI_OUT_INT_MASK        EQU     0x34
PCI_INT_LINE            EQU     0x3C
MAILBOX_0               EQU     0x50
MAILBOX_1               EQU     0x54
MAILBOX_2               EQU     0x58
MAILBOX_3               EQU     0x5C
DOORBELL                EQU     0x60
DOORBELL_SETUP          EQU     0x64
ROM_BYTE_WRITE          EQU     0x68
CHAN_1_BYTE_COUNT       EQU     0x80
CHAN_1_PCI_ADDR         EQU     0x84
CHAN_1_DRAM_ADDR        EQU     0x88
CHAN_1_DESC_PTR         EQU     0x8C
CHAN_1_CONTROL          EQU     0x90
CHAN_2_BYTE_COUNT       EQU     0xA0
CHAN_2_PCI_ADDR         EQU     0xA4
CHAN_2_DRAM_ADDR        EQU     0xA8
CHAN_2_DESC_PTR         EQU     0xAC
CHAN_2_CONTROL          EQU     0xB0
DRAM_BASE_ADDR_MASK     EQU     0x100
DRAM_BASE_ADDR_OFF      EQU     0x104
ROM_BASE_ADDR_MASK      EQU     0x108
DRAM_TIMING             EQU     0x10C
DRAM_ADDR_SIZE_0        EQU     0x110
DRAM_ADDR_SIZE_1        EQU     0x114
DRAM_ADDR_SIZE_2        EQU     0x118
DRAM_ADDR_SIZE_3        EQU     0x11C
I2O_IFH                 EQU     0x120
I2O_IPT                 EQU     0x124
I2O_OPH                 EQU     0x128
I2O_OFT                 EQU     0x12C
I2O_IFC                 EQU     0x130
I2O_OPC                 EQU     0x134
I2O_IPC                 EQU     0x138
SA_CONTROL              EQU     0x13C
PCI_ADDR_EXT            EQU     0x140
PREFETCH_RANGE          EQU     0x144
XBUS_CYCLE              EQU     0x148
XBUS_STROBE             EQU     0x14C
DBELL_PCI_MASK          EQU     0x150
DBELL_SA_MASK           EQU     0x154
UARTDR                  EQU     0x160
UMSEOI                  EQU     0x164
RXSTAT                  EQU     0x164 
H_UBRLCR                EQU     0x168 
M_UBRLCR                EQU     0x16C 
L_UBRLCR                EQU     0x170 
UARTCON                 EQU     0x174 
UARTFLG                 EQU     0x178 
IRQ_STATUS              EQU     0x180 
IRQ_RAW_STATUS          EQU     0x184 
IRQ_ENABLE              EQU     0x188 
IRQ_ENABLE_SET          EQU     0x188 
IRQ_ENABLE_CLEAR        EQU     0x18C 
IRQ_SOFT                EQU     0x190 
FIQ_STATUS              EQU     0x280 
FIQ_RAW_STATUS          EQU     0x284 
FIQ_ENABLE              EQU     0x288 
FIQ_ENABLE_SET          EQU     0x288 
FIQ_ENABLE_CLEAR        EQU     0x28C 
FIQ_SOFT                EQU     0x290 
TIMER_1_LOAD            EQU     0x300 
TIMER_1_VALUE           EQU     0x304 
TIMER_1_CONTROL         EQU     0x308 
TIMER_1_CLEAR           EQU     0x30C 

TIMER_2_LOAD            EQU     0x320 
TIMER_2_VALUE           EQU     0x324 
TIMER_2_CONTROL         EQU     0x328 
TIMER_2_CLEAR           EQU     0x32C 

TIMER_3_LOAD            EQU     0x340 
TIMER_3_VALUE           EQU     0x344 
TIMER_3_CONTROL         EQU     0x348 
TIMER_3_CLEAR           EQU     0x34C 

TIMER_4_LOAD            EQU     0x360 
TIMER_4_VALUE           EQU     0x364 
TIMER_4_CONTROL         EQU     0x368 
TIMER_4_CLEAR           EQU     0x36C 


; /* These are all defined in terms of offsets from DC21285_ARMCSR_BASE */

DC21285_ARMCSR_UART_BASE        EQU     UARTDR
;DC21285_UCR                    EQU     DC21285_ARMCSR_BASE+0x160
;DC21285_UDR                    EQU     DC21285_ARMCSR_BASE+0x164
;DC21285_USR                    EQU     DC21285_ARMCSR_BASE+0x168
        
DC21285_ARMCSR_IRQ_BASE         EQU     IRQ_STATUS
; /* Offsets from IRQ_BASE, or FIQ_BASE (in words) */
INTStatus                       EQU     0
INTRawStatus                    EQU     1
INTEnable                       EQU     2
INTEnableSet                    EQU     2
INTEnableClear                  EQU     3
INTSoft                         EQU     4

DC21285_IRQStatus               EQU     DC21285_ARMCSR_BASE+0x180

DC21285_ARMCSR_TIMER_BASE       EQU     TIMER_1_LOAD
; /* Timer enable bit in control register */
TIMER_CONTROL_ENABLE            EQU     (1 << 7) 

;
; Now define individual registers in the clumps.
;
;
; AMBA port registers.  These offsets and bit settings are taken from
; the AMBA uart (AP13) data sheet.
;
AMBA_UARTDR                     EQU     0x00
AMBA_RXSTAT                     EQU     0x04
AMBA_H_UBRLCR                   EQU     0x08
AMBA_M_UBRLCR                   EQU     0x0C
AMBA_L_UBRLCR                   EQU     0x10
AMBA_UARTCON                    EQU     0x14
AMBA_UARTFLG                    EQU     0x18

AMBA_UARTFLG_TFF                EQU     0x20
AMBA_UARTFLG_RFE                EQU     0x10
AMBA_UARTFLG_TBY                EQU     0x08
AMBA_UARTFLG_TMSK               EQU     (AMBA_UARTFLG_TFF + AMBA_UARTFLG_TBY)

AMBA_UARTCON_I                  EQU     0x04
AMBA_UARTCON_S                  EQU     0x02
AMBA_UARTCON_UE                 EQU     0x01

AMBA_H_UBRLCR_DSS_8             EQU     0x60
AMBA_H_UBRLCR_DSS_7             EQU     0x40
AMBA_H_UBRLCR_DSS_6             EQU     0x20
AMBA_H_UBRLCR_DSS_5             EQU     0x00
AMBA_H_UBRLCR_EF                EQU     0x10
AMBA_H_UBRLCR_SBS               EQU     0x08
AMBA_H_UBRLCR_OES               EQU     0x04
AMBA_H_UBRLCR_PE                EQU     0x02
AMBA_H_UBRLCR_BRK               EQU     0x01

; /* ARM control register constants */
INIT_COMPLETE           EQU     1
PCI_NRESET              EQU     0x200
XCS_2                   EQU     0x40000000
XCS_1                   EQU     0x20000000
XCS_0                   EQU     0x10000000
PCI_CFN                 EQU     0x80000000
ROM_TIMING              EQU     0x0fff0000
ROM_SETUP               EQU     0x04aa0000

; Initial PCI Interrupt ID is 1.
SETUP_PCI_INT_ID        EQU     0x100

; These definitions assume that if either of these are defined,
; then both are 
 IF :DEF: HANDLE_INTERRUPTS_ON_IRQ
  IF HANDLE_INTERRUPTS_ON_IRQ <> 0
ANGEL_INT_ENABLE        EQU     (CSR_BASE + IRQ_ENABLE_SET)
ANGEL_INT_DISABLE       EQU     (CSR_BASE + IRQ_ENABLE_CLEAR)
ANGEL_INT_SOFT          EQU     (CSR_BASE + IRQ_SOFT)
ANGEL_INT_STATUS        EQU     (CSR_BASE + IRQ_STATUS)
ANGEL_INT_RAW           EQU     (CSR_BASE + IRQ_RAW_STATUS)
  ENDIF
 ENDIF
 IF :DEF: HANDLE_INTERRUPTS_ON_FIQ
  IF HANDLE_INTERRUPTS_ON_FIQ <> 0
ANGEL_INT_ENABLE        EQU     (CSR_BASE + FIQ_ENABLE_SET)
ANGEL_INT_DISABLE       EQU     (CSR_BASE + FIQ_ENABLE_CLEAR)
ANGEL_INT_SOFT          EQU     (CSR_BASE + FIQ_SOFT)
ANGEL_INT_STATUS        EQU     (CSR_BASE + FIQ_STATUS)
ANGEL_INT_RAW           EQU     (CSR_BASE + FIQ_RAW_STATUS)
  ENDIF
 ENDIF

; On the verification board we have two UARTS, on the evaluation board
; one AMBA port. When debugging, we may need to access both.
XBAUD_115200            EQU     0x001 
XBAUD_57600             EQU     0x002 
XBAUD_38400             EQU     0x003 
XBAUD_19200             EQU     0x006 
XBAUD_14400             EQU     0x009 
XBAUD_9600              EQU     0x00C 
XBAUD_2400              EQU     0x018 
XBAUD_1200              EQU     0x060    
XDEFAULT_HOST_BAUD      EQU     BAUD_9600
XDEFAULT_OS_BAUD        EQU     BAUD_38400

EB285_USE_UARTS         EQU     0

 IF EB285_USE_UARTS <> 0
UARTS                   EQU     1
; /* Default port to talk to host (via debugger) */
HOST_COMPORT            EQU     COM1Port
; /* Default port for use by Operating System or program */
OS_COMPORT              EQU     COM2Port
DEBUG_COMPORT           EQU     OS_COMPORT

; /* Values to set given baud rates */
BAUD_115200             EQU     XBAUD_115200
BAUD_57600              EQU     XBAUD_57600
BAUD_38400              EQU     XBAUD_38400
BAUD_19200              EQU     XBAUD_19200
BAUD_14400              EQU     XBAUD_14400
BAUD_9600               EQU     XBAUD_9600
BAUD_2400               EQU     XBAUD_2400
BAUD_1200               EQU     XBAUD_1200
DEFAULT_HOST_BAUD       EQU     XDEFAULT_HOST_BAUD
DEFAULT_OS_BAUD         EQU     XDEFAULT_OS_BAUD

;#define IO_READ(p)             (*(unsigned char *)(_MapAddress(p)))
;#define IO_WRITE(p, c)         (*(unsigned char *)(_MapAddress(p)) = c)

 ELSE
UARTS                   EQU     0
; /* Default port to talk to host (via debugger) */
HOST_COMPORT            EQU     (CSR_BASE + DC21285_ARMCSR_UART_BASE)
; /* Default port for use by Operating System or program */
OS_COMPORT              EQU     HOST_COMPORT
; WARNING: COM2Port Needs UART daughtercard to work!!
DEBUG_COMPORT           EQU     COM2Port

; /* Values to set given baud rates for a UART clock frequency of 50 MHz */
BAUD_115200             EQU     0x006 
BAUD_57600              EQU     0x00D 
BAUD_38400              EQU     0x013 
BAUD_19200              EQU     0x028 
BAUD_14400              EQU     0x03C 
BAUD_9600               EQU     0x050 
BAUD_2400               EQU     0x145 
BAUD_1200               EQU     0x28A    
BAUD_110                EQU     0x1BBD
DEFAULT_HOST_BAUD       EQU     BAUD_9600
DEFAULT_OS_BAUD         EQU     DEFAULT_HOST_BAUD

;#define GET_STATUS(p)          (inb( (void *) ((p) + AMBA_UARTFLG) ) & 0xFF)
;#define GET_CHAR(p)            (inb( (void *) ((p) + AMBA_UARTDR) ))
;#define PUT_CHAR(p, c)         (outb(c, (void *)((p) + AMBA_UARTDR)))
;#define IO_READ(p)             (AMBA_REG_READ(p))
;#define IO_WRITE(p, c)         (AMBA_REG_WRITE(p, c))
;#define RX_DATA(s)             (((s) & AMBA_UARTFLG_RFE) == 0)
;#define TX_READY(s)            (((s) & AMBA_UARTFLG_TFF) == 0)
;#define TX_EMPTY(p)            ((GET_STATUS(p) & AMBA_UARTFLG_TMSK) == 0)

; /* Used by Angel for interrupts */
;#define READ_INTERRUPT(port)   ( AMBA_REG_READ(ANGEL_INT_STATUS) )
;/* Enable & disable write the bit to different addresses */
;#define ENABLE_INT(v, b, p)    ( AMBA_REG_WRITE(ANGEL_INT_ENABLE, b) )
;#define DISABLE_INT(v, b, p)   ( AMBA_REG_WRITE(ANGEL_INT_DISABLE, b) )
;#define ENABLE_INTERRUPT(p)    /* Do nothing */
NO_INTERRUPT            EQU     0x00
TX_INTERRUPT            EQU     0x08
RX_INTERRUPT            EQU     0x04

TX_ENABLE               EQU     TX_INTERRUPT
RX_ENABLE               EQU     RX_INTERRUPT

 ENDIF

;#define AMBA_REG_READ(r)       *((unsigned int *)(r))
;#define AMBA_REG_WRITE(r, v)   *(unsigned int *)((unsigned int)(r)) = v

; /* Keep aliases for XBus UART */
;#define XIO_READ(p)            (*(unsigned char *)(_MapAddress(p)))
;#define XIO_WRITE(p, c)                (*(unsigned char *)(_MapAddress(p)) = c)

Baud1200        EQU     XBAUD_1200
Baud2400        EQU     XBAUD_2400
Baud9600        EQU     XBAUD_9600
Baud19200       EQU     XBAUD_19200
Baud38400       EQU     XBAUD_38400
Baud57600       EQU     XBAUD_57600
Baud115200      EQU     XBAUD_115200

; Purely for compatability with ANGEL (defined in uart.h in uHAL):
InterruptId                     EQU     2
; Purely for compatability with uHAL (defined in superio_h.s in ANGEL):
EnableInt                       EQU     0x04
CntlModem                       EQU     0x10

 IF :DEF: LOGTERM_DEBUGGING
; For Angel only, need to include this header file in platform.h
;#include	"angel285.h"
 ENDIF

; timer stuff:
; On EB285 there is an external clock which ticks at 3.68Mhz. This is wired
; into the DC21285 chip's timer #3. There are 24 bits of counter that we can
; set to divide this external timer down. Each external tick is 3.68*(10**-6)
; seconds long, 1mSEC needs counter of 3680 (every 3680 external ticks, we
; get one clock interrupt).

OS_TIMER                        EQU     3
MAX_TIMER                       EQU     4
; If DEMON were ever to be ported to the EBSA-285, or ANGEL needed a timer,
; it would use MAX_TIMER.
HOST_TIMER                      EQU     MAX_TIMER

mSEC_1                          EQU     3680    
mSEC_5                          EQU     (mSEC_1 * 5)
mSEC_10                         EQU     (mSEC_1 * 10)
SEC_1                           EQU     (mSEC_1 * 1000)

MAX_PERIOD                      EQU     0xFFFF          

; Timer definitions.            
; The irq numbers of the individual timers
;#define TIMER_VECTORS  { 0, 4, 5, 6, 7 }

; set cycle length = 2 for all devices, strobe shift divisor = 1 (2)
; Octal 022222
; SuperIO drives interrupts active high, tell 21285 to do the same
INIT_XBUS_CYCLE         EQU     0x100016DB
INIT_XBUS_STROBE        EQU     0xfcfcfcfc

; xcs<2> = output (softIO, leds and jumpers), xcs<1> = output (SuperIO), 
; xcs<0> = input (interrupt)
XBUS_INIT               EQU     (XCS_2 | XCS_1)
; Note: This is a bodge to get around an assembler limitation. XBUS_BITS
; should really be:
;XBUS_BITS              EQU     (XCS_2 | XCS_1 | XCS_0)
; .. but the assembler complains!
XBUS_BITS               EQU     (XBUS_INIT | XCS_0)
SA_CTL_INIT             EQU     (XBUS_INIT | ROM_SETUP)
SA_CTL_COMPLETE         EQU     (INIT_COMPLETE | PCI_NRESET)

; PCI Command Register
PCI_IO_ENABLE           EQU     0x0001
PCI_MEM_ENABLE          EQU     0x0002
PCI_MASTER_ENABLE       EQU     0x0004
PCI_MEM_WRINVALID       EQU     0x0010
PCI_FAST_B2B            EQU     0x0200
; Another workaround the Assembler limitation. PCI_CFN_INIT should be:
;PCI_CFN_INIT           EQU     (PCI_IO_ENABLE | PCI_MEM_ENABLE | PCI_MASTER_ENABLE | PCI_MEM_WRINVALID)
PCI_CFN_ENABLE          EQU     (PCI_IO_ENABLE | PCI_MEM_ENABLE)
PCI_MAS_ENABLE          EQU     (PCI_MEM_WRINVALID | PCI_MASTER_ENABLE)
PCI_CFN_INIT            EQU     (PCI_CFN_ENABLE | PCI_MAS_ENABLE)

; PCI Outbound Interrupt Status/Mask
PCI_OUT_DOORBELL        EQU     0x04
PCI_OUT_POSTLIST        EQU     0x08
NO_OUTBOUND_INTS        EQU     (PCI_OUT_DOORBELL | PCI_OUT_POSTLIST)

; 21285 puts it's DRAM memory in this Base Address Register
HOST_MEMBAR             EQU     2       ; Standard nomenclature


; The following are used in the startup code for the EB285. There are two
; DIMMs of RAM on the EB285, and there may be DRAM in  either block. The
; DRAM is initialised by the startup code.
LowRAMBase              EQU     0

MAX_SDRAM               EQU     4

; /* See 21285 Page 7-31 of data sheet: */
DC21285_Trp1            EQU     0x0
DC21285_Trp2            EQU     0x1
DC21285_Trp3            EQU     0x2
DC21285_Trp4            EQU     0x3
DC21285_Tdal2           EQU     (0x0 << 2)
DC21285_Tdal3           EQU     (0x1 << 2)
DC21285_Tdal4           EQU     (0x2 << 2)
DC21285_Tdal5           EQU     (0x3 << 2)
DC21285_Trcd2           EQU     (0x2 << 4)
DC21285_Trcd3           EQU     (0x3 << 4)
DC21285_Tcas2           EQU     (0x2 << 6)
DC21285_Tcas3           EQU     (0x3 << 6)
DC21285_Trc4            EQU     (0x1 << 8)
DC21285_Trc5            EQU     (0x2 << 8)
DC21285_Trc6            EQU     (0x3 << 8)
DC21285_Trc7            EQU     (0x4 << 8)
DC21285_Trc8            EQU     (0x5 << 8)
DC21285_Trc9            EQU     (0x6 << 8)
DC21285_Trc10           EQU     (0x7 << 8)
DC21285_Tcd0            EQU     (0x0 << 11)
DC21285_Tcd1            EQU     (0x1 << 11)
DC21285_Parity0         EQU     (0x0 << 12)
DC21285_Parity1         EQU     (0x1 << 12)
; Tref, No Parity, CDT=0, Trc=4, Tcas=2, Trcd=2, Tdal(Trp+Trdl)=3, Trp=2
;       0          0      010      10      01                01

saTrp                   EQU     DC21285_Trp2
saTdal                  EQU     DC21285_Tdal3
saTrcd                  EQU     DC21285_Trcd2
saTcas                  EQU     DC21285_Tcas2
saTrc                   EQU     DC21285_Trc4
saTcd                   EQU     DC21285_Tcd0
saParity                EQU     DC21285_Parity0

SDRAM_MODEBITS          EQU     (saParity + saTcd + saTrc + saTcas + saTrcd + saTdal + saTrp)

; /* Tref=1 */
INITIAL_TREF            EQU     (0x01 << 16)    ; Top half of word
INITIAL_TIMING          EQU     (INITIAL_TREF + SDRAM_MODEBITS)

; /* Tref=1a (64 ms x 4096 rows) = 15.xx us */
SETUP_TREF              EQU     (0x1a << 16)    ; Top half of word
SETUP_TIMING            EQU     (SETUP_TREF + SDRAM_MODEBITS)

REFRESH_COUNT           EQU     32              ; No. of cycles scaled per wait
REFRESH_WAIT            EQU     8               ; Max. wait per SDRAM

; /* Interrupt Flags */
VALID_INTMASK           EQU     0xf3c3fffe
SOFT_INT                EQU     0x00000002
AMBACOM1_RX             EQU     0x00000004
AMBACOM1_TX             EQU     0x00000008
TIMER1_INT              EQU     0x00000010
TIMER2_INT              EQU     0x00000020
TIMER3_INT              EQU     0x00000040
TIMER4_INT              EQU     0x00000080
IRQ_IN3_INT             EQU     0x00000400      ; StrongARM 3.186MHz Xtal here
XSCOM1_INT              EQU     0x00001000      ; XBUS_DEV_1 interrupts on xcs_l[0]
XSCOM2_INT              EQU     0x00001000
DOORBELL_INT            EQU     0x00008000
DMA1_INT                EQU     0x00010000
DMA2_INT                EQU     0x00020000
MEM_PARITY_INT          EQU     0x01000000
I2O_INPOST_INT          EQU     0x02000000
RAM_PARITY_INT          EQU     0x10000000
PCI_MASTER_INT          EQU     0x20000000
PCI_TARGET_INT          EQU     0x40000000
PCI_PARITY_INT          EQU     0x80000000
 IF :DEF: AMBA_SERIAL
ANGEL_INTS              EQU     (AMBACOM1_TX | AMBACOM1_RX)
 ELSE
ANGEL_INTS              EQU     XSCOM1_INT
 ENDIF

; /* Auto-select the appropriate value: */
 IF saTcas <> DC21285_Tcas2
CAS_OFFSET              EQU     0xc8            ; If CAS Latency is 3 cycles
 ELSE
CAS_OFFSET              EQU     0x88            ; If CAS Latency is 2 cycles
 ENDIF

; /* SD_OFFSET is the difference between SDRAM Array Register banks */
SD_OFFSET               EQU     (DC21285_DRAM_A1MR - DC21285_DRAM_A0MR)

; /* Row/Col Mux modes shifted into correct postion (bits 6, 5 and 4) */
MUX_MODE0               EQU     0x00
MUX_MODE1               EQU     0x10
MUX_MODE2               EQU     0x20
MUX_MODE3               EQU     0x30
MUX_MODE4               EQU     0x40

DRAM_4BANKS             EQU     0x08
DRAM_2BANKS             EQU     0x00
MUX_MASK                EQU     0x78

; /* Bit patterns for SDRAM memory sizes */
SDSIZE_1M               EQU     0x01
SDSIZE_2M               EQU     0x02
SDSIZE_4M               EQU     0x03
SDSIZE_8M               EQU     0x04
SDSIZE_16M              EQU     0x05
SDSIZE_32M              EQU     0x06
SDSIZE_64M              EQU     0x07

; /* 8MB bit pattern for Verification Vehicle base memory */
VV_BASE_MEM             EQU     (SDSIZE_8M | MUX_MODE1)

ALLBITS                 EQU     0xffffffff

uHAL_MEMORY_SIZE        EQU     (DRAM_DEFAULT)

MAXIRQNUM               EQU     31
MAXFIQNUM               EQU     31
MAXSWINUM               EQU     31

NR_IRQS                 EQU     (MAXIRQNUM + 1)

; StrongARM Flash block definitions. Used by the primary bootloader to
; allow one of several images to be executed at boot-up.
FLASH_BLOCK_SIZE        EQU     SZ_256K
IMAGE_SWITCH            EQU     XBUS_DEV_2      ; Anywhere in XCS2 space

IMAGE_MASK              EQU     0x0F
BENCH_BIT               EQU     0x40
        
; /* Flash header definition */
FH_TYPE                 EQU     0x0
FH_NUMBER               EQU     0x4
FH_SIG_START            EQU     0x5
FH_SIG_END              EQU     0x8
FH_MAP                  EQU     0x8
FH_CHECKSUM             EQU     0xC
FH_LENGTH               EQU     0x10
FH_NAME                 EQU     0x14
FH_NAME_END             EQU     0x24
FH_BOOTFLAGS            EQU     0x24
FH_AIF                  EQU     0x40
FH_END                  EQU     0xC0

FH_NOBOOTMASK           EQU     1
                
; /* AIF header definition */
AIF_DECOMPRESS          EQU     0x0
AIF_SELF_RELOC          EQU     0x4
AIF_INIT                EQU     0x8
AIF_ENTRY               EQU     0xC
AIF_EXIT                EQU     0x10
AIF_RO_SIZE             EQU     0x14
AIF_RW_SIZE             EQU     0x18
AIF_DEBUG_SIZE          EQU     0x1C
AIF_ZERO_INIT_SIZE      EQU     0x20
AIF_DEBUG_TYPE          EQU     0x24
AIF_BASE                EQU     0x28
AIF_WORKSPACE           EQU     0x2C
AIF_ADDR_MODE           EQU     0x30
AIF_DATA_BASE           EQU     0x34
AIF_DEBUG_INIT          EQU     0x40
AIF_ZINIT_CODE          EQU     0x44
AIF_END                 EQU     0x80
        
FH_AIF_BASE             EQU     ((FH_AIF)+(AIF_BASE))
FH_AIF_ENTRY            EQU     ((FH_AIF)+(AIF_ENTRY))


 ENDIF

        END
