/***********************************************************************
 * Copyright  Intel Corporation 1998.  All rights reserved.
 ***********************************************************************

 This file contain somes stubs and stuff to allow linking with uHAL
 without missing link errors
        Geoff Smith.
        
 Functions:
        printf - prints nothing at all.
        
 This also brings devnode.c and pci.c in as include files, this avoids
 limitations in the ARM SDT so only 1 project file is needed for
 ethernet and serial only versions of Angel.

 ***********************************************************************/

#include "devconf.h"

#if ETHERNET_SUPPORTED 

#include "system.h"
#include "net_buff.h"
#include "platform.h"

extern U32 SAiv_PciSizes[]; // From driver.s

extern U32 SAiv_PciMasks[]; // from driver.s



#if CACHE_SUPPORTED
/*
        Flush the area shared with PCI/Ethernet 
*/
void flush_memory(void) 
{
        char *ptr_start, *ptr_end;
        
        ptr_start = (char*)EthernetBufferBase;          
        ptr_end = ptr_start + PCI_DRAMSIZE;
        Cache_IBR(ptr_start, ptr_end+1);
}
#else
void flush_memory(void) 
{
return;
}

#endif

void printf(char *format, ...)
{
        
        (*format == *format)?0:0;
}

void SAr_printf(char *format, ...)
{
        
        (*format == *format)?0:0;
}


/*
 * routine:     SAir_PciHost()
 *
 * parameters:  void
 *
 * description: This routine determines if the target is PCI Host.
 *
 * calls:       none
 *
 * returns:     FALSE if not PCI Host (or PCI not defined for this target) or
 *              handle of devnode for this 21285
 *
 */
U32 SAr_PciHost(void)
{
U8      *controlReg = (U8 *)DC21285_ARMCSR_BASE + SA_CONTROL ;
U32     centralFunction, standAlone ;

        centralFunction = *(U32 *)controlReg & PCI_CFN ;
        standAlone = (*((U32 *)IMAGE_SWITCH) & BENCH_BIT) ? TRUE : FALSE ;

        if (centralFunction && !standAlone)
                return (MAKE_HANDLE(PCI_TYPE, 0, 0, 0)) ;
        else
        
return (FALSE) ;
}

/*
        Is the ebsa285 in a PCI backplane?
        Reurns TRUE if it is
*/
U32 SAr_PciBenchMode(void)
{
        return (*((U32 *)IMAGE_SWITCH) & BENCH_BIT) ? TRUE : FALSE ;
}



void SAir_AddFixedDevNodes(void)
{
}

U32 SAir_PciMakeConfigAddress(U32 devHandle, U32 offset)
{
U32     bus, device, function ;
U32     address ;

        /* extract the bus, device and function from the handle */
        bus = (U32)GET_BUS(devHandle) ;
        device = (U32)GET_DEVICE(devHandle) ;
        function = (U32)GET_FUNCTION(devHandle) ;
        /* DC21285 uses different zones for type 0 and 1 configuration cycles */
        if (bus == 0) {
                if (device == 0) {
                        /* Read from our 21285 */
                        address = (U32) DC21285_ARMCSR_BASE ;
                }
                else {
                        address = (U32) DC21285_PCI_TYPE_0_CONFIG ;
                        address |= (U32)(1 << (device + 11)) ;
                }
        } else {
                address = (U32) DC21285_PCI_TYPE_1_CONFIG ;
                address |= (device << 11) ;
                address |= (bus << 16) ;
        }

        /* Note this is NOT shifted left by 2 as per the above comment */
	/* The following mask has been changed from 3f to ff to ensure 
	   all offsets are catered for 3f precludes 40 which is required 
	   in setting up the cfdd config register - dawb */
        address |= offset & 0xff ;
        address |= ((function & 0x03) << 8) ;

        return address ;

}


#include "devnode.c"
#include "pci.c"



#endif
