#ifndef __ARP_H_LOADED
#define __ARP_H_LOADED
/****************************************************************************
**                                                                          *
**  Copyright  1997						     								*
**  by Digital Equipment Corporation, Maynard, Massachusetts.		     	*
**                                                                          *
**  All Rights Reserved							     						*
**                                                                          *
**  Permission  is  hereby  granted  to  use, copy, modify and distribute   *
**  this  software  and  its  documentation,  in  both  source  code  and   *
**  object  code  form,  and without fee, for the purpose of distribution   *
**  of this software  or  modifications  of this software within products   *
**  incorporating  an  integrated   circuit  implementing  Digital's  		*
**  StrongARM architecture,  regardless  of the  source of such integrated 	*
**	circuit, provided that the  above copyright  notice and this permission *
**  notice appear  in  all copies,  and  that  the  name  of  Digital  		*
**  Equipment Corporation  not  be  used  in advertising or publicity 		*
**  pertaining to distribution of the  document  or  software without 		*
**  specific, written prior permission.							            *
**                                                                          *
**  Digital  Equipment  Corporation   disclaims  all   warranties  and/or   *
**  guarantees  with  regard  to  this  software,  including  all implied   *
**  warranties of fitness for  a  particular purpose and merchantability,   *
**  and makes  no  representations  regarding  the use of, or the results   *
**  of the use of, the software and documentation in terms of correctness,  *
**  accuracy,  reliability,  currentness  or  otherwise;  and you rely on   *
**  the software, documentation and results solely at your own risk.	    *
**                                                                          *
**  StrongARM is a trademark of Digital Equipment Corporation               *
**                                                                          *
*****************************************************************************/

/* 
 * $Id: arp.h,v 2.0 1997/07/23 13:26:04 smithg Exp $
 */

/*
 * MODULE DESCRIPTION:
 * 
 *     Arp interface for EB64 monitor
 *
 * HISTORY:
 * 
 * $Log: arp.h,v $
 * Revision 2.0  1997/07/23  13:26:04  smithg
 * ebsa285 revisions
 *
 * Revision 1.2  1997/07/23 10:57:10  smithg
 * Header updated
 *
 * Revision 1.1  1997/07/14 14:45:49  smithg
 * Initial revision
 *
 * Revision 1.3  1997/07/04 10:03:34  smithg
 * changed to unix format from dos format
 *
 * Revision 1.2  1997/07/03 10:32:32  smithg
 * system.h includes removed
 *
 * Revision 1.1  1996/03/28 11:14:54  rusling
 * Initial revision
 *
 * Revision 1.1  1996/01/09  14:26:48  berent
 * Initial revision
 *
 * Revision 1.6  1994/08/05  20:13:47  fdh
 * Updated Copyright header and RCS $Id: identifier.
 *
 * Revision 1.5  1994/06/28  20:08:21  fdh
 * Modified filenames and build precedure to fit into a FAT filesystem.
 *
 * Revision 1.4  1994/06/20  14:18:59  fdh
 * Fixup header file preprocessor #include conditionals.
 *
 * Revision 1.3  1994/01/21  09:45:59  rusling
 * Added #ifdef <filename>_H around the module.
 * Additionally, any included files are not ifdef'd
 * *before* they're included (ie address.h).
 *
 * Revision 1.2  1994/01/20  15:46:47  rusling
 * Added new routine, arp_show(), to print out
 * the contents of the current arp table.
 *
 * Revision 1.1  1993/08/06  10:25:46  berent
 * Initial revision
 *
 *
 */

#include "address.h"


/* arp_init - initialises the arp module; and registers it with the
 * ethernet protocol handler.
 *
 * Argument:
 *    device_no - device number on which arp should be registered.
 */
extern void arp_init(int device_no);

/* arp_set_device_addr - tells arp a device's ip address.
 *
 * Arguments:
 *     device_no - device number
 *     ip        - ip address
 */
extern void arp_set_device_addr(int device_no,ip_addr ip);
 
/* arp_resolve - converts an ip address into an ethernet address
 *
 * Arguments:
 *     device_no - device on which the message is to be sent
 *     ip        - ip address
 *     mac       - returned ethernet MAC address
 *
 * Returned value:
 *     TRUE      - address resolved; eaddr valid
 *     FALSE     - Unable to resolve address.
 */
extern int arp_resolve(int device_no, ip_addr ip, mac_addr mac);

/* arp_init_module - initialise the module 
 */
extern void arp_init_module(void);
/*
 * arp_show - show all arp entries
 */
extern void arp_show(void);

/* This adds mac/ip pair into the arp table
	I recommend that this is used by IP whenever a
	packet is received to prevent ARP having to hassle the network */
extern void arp_add(int device_no, ip_addr ip, mac_addr mac);

/*
  This adds a mac/ip pair into position zerp of the ARP table.
  Postion 0 is only overwritted by this function.
*/
extern int arp_force_host_address(int device_no, ip_addr ip, mac_addr mac);

/*
	ARP input calls this is it receives a RARP response
*/
extern void rarp_input(void * protocol_data,
		      int device_no,
		      unsigned char* packet,
		      int size,
		      mac_addr source,
		      unsigned char* frame_buffer,
		      unsigned char* * balance_buffer);
			  
extern int rarp_init(int);

#endif /* __ARP_H_LOADED */
