/****************************************************************************
**                                                                          *
**  Copyright  1997						     								*
**  by Digital Equipment Corporation, Maynard, Massachusetts.		     	*
**                                                                          *
**  All Rights Reserved							     						*
**                                                                          *
**  Permission  is  hereby  granted  to  use, copy, modify and distribute   *
**  this  software  and  its  documentation,  in  both  source  code  and   *
**  object  code  form,  and without fee, for the purpose of distribution   *
**  of this software  or  modifications  of this software within products   *
**  incorporating  an  integrated   circuit  implementing  Digital's  		*
**  StrongARM architecture,  regardless  of the  source of such integrated 	*
**	circuit, provided that the  above copyright  notice and this permission *
**  notice appear  in  all copies,  and  that  the  name  of  Digital  		*
**  Equipment Corporation  not  be  used  in advertising or publicity 		*
**  pertaining to distribution of the  document  or  software without 		*
**  specific, written prior permission.							            *
**                                                                          *
**  Digital  Equipment  Corporation   disclaims  all   warranties  and/or   *
**  guarantees  with  regard  to  this  software,  including  all implied   *
**  warranties of fitness for  a  particular purpose and merchantability,   *
**  and makes  no  representations  regarding  the use of, or the results   *
**  of the use of, the software and documentation in terms of correctness,  *
**  accuracy,  reliability,  currentness  or  otherwise;  and you rely on   *
**  the software, documentation and results solely at your own risk.	    *
**                                                                          *
**  StrongARM is a trademark of Digital Equipment Corporation               *
**                                                                          *
*****************************************************************************//* 
 * $Id: edriver.h,v 2.0 1997/07/23 13:26:04 smithg Exp $
 */

/*
 * MODULE DESCRIPTION:
 * 
 *     Parameters used to service Ethernet device drivers on the EB64
 *
 * HISTORY:
 * 
 * $Log: edriver.h,v $
 * Revision 2.0  1997/07/23  13:26:04  smithg
 * ebsa285 revisions
 *
 * Revision 1.2  1997/07/23 10:57:10  smithg
 * Header updated
 *
 * Revision 1.1  1997/07/14 14:45:49  smithg
 * Initial revision
 *
 * Revision 1.3  1997/07/04 10:03:34  smithg
 * changed to unix format from dos format
 *
 * Revision 1.2  1997/07/03 10:32:32  smithg
 * system.h includes removed
 *
 * Revision 1.1  1996/03/28 11:14:54  rusling
 * Initial revision
 *
 * Revision 1.1  1996/01/09  14:26:48  berent
 * Initial revision
 *
 * Revision 1.3  1994/08/05  20:17:17  fdh
 * Updated Copyright header and RCS $Id: identifier.
 *
 * Revision 1.2  1994/06/28  20:11:47  fdh
 * Modified filenames and build precedure to fit into a FAT filesystem.
 *
 * Revision 1.1  1993/08/06  10:12:44  berent
 * Initial revision
 *
 *
 */

#define ETHER_DRIVER_H

#ifndef ADDRESSES_H
#include "address.h"
#endif

#include "netparam.h"	/* source of MAX_DEVICES constant */


/* Ethernet and IEEE 802.3 packets have the following format for
 * the MAC (Media Access Control) layer
 *
 *     Preamble (supplied by physical device) 7 octets (0x55) for carrier
 *     recover plus one octet (0xD5) Start_of_Frame_Delimiter
 *
 *     6 octet destination address
 *
 *     6 octet source address
 *
 *     2 octet Protocol_type field (Ethernet) or Length field (802.3)
 *             Valid protocol types are all illegal lengths ie. >1500
 *
 *     48-1500 octet data field including padding if necessary
 *
 *     4 octet frame check sequence (FCS)
 *
 * All octets are sent LSB first "on the wire"
 *
 * If the first bit of the destination address sent is "1", it is known
 * as a multicast address, or group address. The "all 1's" is a special
 * case known as the broadcast address
 *
 * For information - the 2nd bit is a "local bit" used by protocols which
 * define local MAC address spaces eg. DECnet phase IV.
 */

#define MAX_ENET_PACKET 1514     /* 6+6+2+1500; does not include the FCS */
#define MIN_ENET_PACKET 60       /* 6+6+2+46; does not include the FCS */

 
         /* Define the MAC header structure for Ethernet */

typedef struct{
  mac_addr destination;
  mac_addr source;
  ethernet_protocol_id protocol;
} ehternet_header_t;

#define ETHERNET_HEADER_SIZE	14
