/****************************************************************************
**                                                                          *
**  Copyright  1997						     								*
**  by Digital Equipment Corporation, Maynard, Massachusetts.		     	*
**                                                                          *
**  All Rights Reserved							     						*
**                                                                          *
**  Permission  is  hereby  granted  to  use, copy, modify and distribute   *
**  this  software  and  its  documentation,  in  both  source  code  and   *
**  object  code  form,  and without fee, for the purpose of distribution   *
**  of this software  or  modifications  of this software within products   *
**  incorporating  an  integrated   circuit  implementing  Digital's  		*
**  StrongARM architecture,  regardless  of the  source of such integrated 	*
**	circuit, provided that the  above copyright  notice and this permission *
**  notice appear  in  all copies,  and  that  the  name  of  Digital  		*
**  Equipment Corporation  not  be  used  in advertising or publicity 		*
**  pertaining to distribution of the  document  or  software without 		*
**  specific, written prior permission.							            *
**                                                                          *
**  Digital  Equipment  Corporation   disclaims  all   warranties  and/or   *
**  guarantees  with  regard  to  this  software,  including  all implied   *
**  warranties of fitness for  a  particular purpose and merchantability,   *
**  and makes  no  representations  regarding  the use of, or the results   *
**  of the use of, the software and documentation in terms of correctness,  *
**  accuracy,  reliability,  currentness  or  otherwise;  and you rely on   *
**  the software, documentation and results solely at your own risk.	    *
**                                                                          *
**  StrongARM is a trademark of Digital Equipment Corporation               *
**                                                                          *
*****************************************************************************/
#include "platform.h"
#include "net_buff.h"
#include "ether.h"
#include "netparam.h"
#include "arp.h"
#include "ethernet.h"
#include "ip.h"
#include "support.h"
#include "debugio.h"
#include "errors.h"
#include "devconf.h"


#if (ETHERNET_SUPPORTED == 1)
#if (USE_RARP == 1)


/*
 * Protocol structures and field values
 * ====================================
 */

/* 
	RARP protocol fields. 
 */
 
#define AR_HRD_OFFSET 0                  /* Hardware address space - must be Ethernet address space */
#define AR_PRO_OFFSET (AR_HRD_OFFSET+2)  /* Protocol address space - must be IP; value is ethernet protocol id
					   					 of IP (0x0800) */
#define AR_HLN_OFFSET (AR_PRO_OFFSET+2)  /* Byte length of each hardware address; will be 6 */
#define AR_PLN_OFFSET (AR_HLN_OFFSET+1)  /* Byte length of each protocol address; will be 4 */
#define AR_OP_OFFSET  (AR_PLN_OFFSET+1)   /* opcode */
#define AR_SHA_OFFSET (AR_OP_OFFSET +2)  /* Source hardware address */
#define AR_SPA_OFFSET (AR_SHA_OFFSET+MAC_ADDRESS_SIZE)  /* Source IP address */
#define AR_THA_OFFSET (AR_SPA_OFFSET+IP_ADDRESS_SIZE)   /* Target hardware address (if known) */
#define AR_TPA_OFFSET (AR_THA_OFFSET+MAC_ADDRESS_SIZE)  /* Target protocol address */
#define AR_PACKET_SIZE (AR_TPA_OFFSET+IP_ADDRESS_SIZE)  /* Total size of arp packets */


/*
 * Constants used in the ARP protocol 
 */

/* Op codes; defined in RFC 903 */
static char const ar_op_rev_request[2] = {0x00, 0x03};
static char const ar_op_rev_reply[2] = {0x00, 0x04};

/* Ethernet hardware type - see RFC 1340 */
extern char const arp_hardware_ether[];

extern char client_ipaddr[]; /* Its declared in arm_ether.c */

/*
 * Internal functions
 * ==================
 */

/* This routine starts the specified timer hardware. */
void start_timer(void)
{
unsigned int	value ;
unsigned char *counter;
unsigned char *ctrl_word = (unsigned char*) PIT_CTLW;
					

	/* Disable interrupts from the SA's viewpoint */
	*(volatile char*)IRQ_MCLR_REG = PIT1_INTERRUPT_MASK;
	*(volatile unsigned char*)ctrl_word = 0x70; /* cnt 1,r/w 11,mode 0, 16-bit */

	/* Determine which value to write */
	value = mSEC_5;
	counter = (unsigned char *)PIT_CNT1 ;
	/* Setting the 2nd byte of the count starts the timer */
	*(volatile unsigned char *)counter = (unsigned char)(value & 0xff) ;
	*(volatile unsigned char *)counter = (unsigned char)((value >> 8) & 0xff) ;
}

int timer_expired(void)
{
	if ((*(volatile char*)IRQ_RAW_REG) & PIT1_INTERRUPT_MASK)
		return TRUE;	
	else
		return FALSE;
	
}

/*
	rarp_build_packet
*/
static unsigned char* rarp_build_packet(int device_no, unsigned char *frame_buffer)
{
    mac_addr my_mac_address;
    unsigned char* arp_packet;	
	int i;
	
	arp_packet = (frame_buffer + ethernet_data_offset());
    /*
     * Set up a request reverse packet
     */
    *(unsigned char *)(arp_packet+AR_HRD_OFFSET) = arp_hardware_ether[0];
    *(unsigned char *)(arp_packet+AR_HRD_OFFSET+1)= arp_hardware_ether[1];
    for(i=0; i<ETHERNET_PROTOCOL_ID_SIZE; i++)
		*(unsigned char *)(arp_packet+AR_PRO_OFFSET+i) = ether_protocol_ip[i];
    *(unsigned char *)(arp_packet+AR_HLN_OFFSET)= MAC_ADDRESS_SIZE;
    *(unsigned char *)(arp_packet+AR_PLN_OFFSET)= IP_ADDRESS_SIZE;
    *(unsigned char *)(arp_packet+AR_OP_OFFSET) = ar_op_rev_request[0];
    *(unsigned char *)(arp_packet+AR_OP_OFFSET + 1)= ar_op_rev_request[1];
    
    /* Get the device's MAC address and copy it in to the source*/
    ether_device_get_hw_address(device_no, my_mac_address);
    for(i=0; i<MAC_ADDRESS_SIZE; i++)
		*(unsigned char *)(arp_packet+AR_SHA_OFFSET+i) = my_mac_address[i];
	/* Blank the Sender IP address (not strictly necessary) */
    for(i=0; i<IP_ADDRESS_SIZE; i++)
		*(unsigned char *)(arp_packet+AR_SPA_OFFSET+i) = 0;
	/* and copy mac to the target too, this is where the reply reverse will go to. */	
    for(i=0; i<MAC_ADDRESS_SIZE; i++)
		*(unsigned char *)(arp_packet+AR_THA_OFFSET+i) = my_mac_address[i];
	/* And blank target IP */
    for(i=0; i<IP_ADDRESS_SIZE; i++)
		*(unsigned char *)(arp_packet+AR_TPA_OFFSET+i) = 0;
	
	return arp_packet;
}


/*
 * Arguments:
 *     device_no - which device to query
 *     ip        - ip address required
 *	Return:
 * 		0 - success
 *		-ve - error
 */
 
static int rarp_query_network(int const device_no)
{
	unsigned char *frame_buffer, *balance_buffer;
    unsigned char* arp_packet;
    int err, status = 0;
	unsigned int OSTick;
	int done = 1;
	
	if (NULL == (frame_buffer = framemalloc(ETHER_BUFFER_SIZE))) {
		return -1;
	}
	
    /*
     * Now write/read packets for a while.
     */

	OSTick = 0;
	while((OSTick < 1099) && (status == 0) ) {
		int i;
		
			
		if (status)
			break;	
		
		if ((done != OSTick) && (OSTick % 300) == 0) {
			/*
		     * Transmit the packet.
		     */
			done = OSTick; 
			arp_packet = rarp_build_packet(device_no, frame_buffer);			 
			err = ethernet_write(device_no,
					    arp_packet,
					    broadcast_mac_address,
					    ether_protocol_arp,
					    AR_PACKET_SIZE,
					    frame_buffer,
					    &balance_buffer);
			frame_buffer = balance_buffer; 			
		}
	
		if (timer_expired())  {
			start_timer();
			OSTick++;
		}
		ethernet_process_one_packet(device_no, frame_buffer, &balance_buffer);
		frame_buffer = balance_buffer;
		for (i=0;i<IP_ADDRESS_SIZE;i++) 
			status += client_ipaddr[i];	// Do I know my address yet?
			
		if (status)
			break;	
	}
	
	framefree(frame_buffer);	
	
	if (status)	{
		return 0;
	}
	else {
		return ERR_RARPNOTRESOLVED;	
	}

}

/* arp_input - arp received packet processor.
 *
 * Arguments:
 *     protocol_data - protocol data passed to module when protocol registered; not used
 *     device_no     - device on which packet was received;
 *     packet        - packet, with ethernet header removed;
 *     size          - size of packet;
 *     source        - source MAC address of packet;
 *     frame_buffer  - original buffer containing packet.
 *     balance_buffer- buffer returned to maintain buffer balance buffer.
 *
 * Description:
 *     Processes a received RARP packet. 
 */
 
 void rarp_input(void * protocol_data,
		      int device_no,
		      unsigned char* packet,
		      int size,
		      mac_addr source,
		      unsigned char* frame_buffer,
		      unsigned char* * balance_buffer)
{
	int i;
	mac_addr my_mac;
	ip_addr ip;


	*balance_buffer = frame_buffer;
	/* check its a reverse response */	
	if ((*(unsigned char*)(packet + AR_OP_OFFSET) != ar_op_rev_reply[0]) ||
		(*(unsigned char*)(packet + AR_OP_OFFSET + 1) != ar_op_rev_reply[1]))
		return;
	
	/* Check its for me... */
    ether_device_get_hw_address(device_no, my_mac);
    for(i=0; i<MAC_ADDRESS_SIZE; i++) {
		if (*(unsigned char*)(packet + AR_THA_OFFSET + i) != my_mac[i]) {
			return;
		}
	}

	
	/* Okay, its mine, its a RARP response copy out IP address */	

   	__rt_memcpy(ip,(unsigned char*)(packet + AR_TPA_OFFSET), IP_ADDRESS_SIZE); 	
	ip_set_device_addr(device_no, ip);
	
}

/*
 * Externally callable functions
 * =============================
 */


/* !!!! Must start after ARP and net_buff !!!!
 * Argument:
 *    device_no - device number on which arp should be registered.
 */
int rarp_init(int device_no)
{
	/* Discover my own IP address before we get too far so
		timers will not be affected */
   return rarp_query_network(device_no);   		
}

#endif
#endif // USE_RARP
