/****************************************************************************
**                                                                          *
**  Copyright  1997						     								*
**  by Digital Equipment Corporation, Maynard, Massachusetts.		     	*
**                                                                          *
**  All Rights Reserved							     						*
**                                                                          *
**  Permission  is  hereby  granted  to  use, copy, modify and distribute   *
**  this  software  and  its  documentation,  in  both  source  code  and   *
**  object  code  form,  and without fee, for the purpose of distribution   *
**  of this software  or  modifications  of this software within products   *
**  incorporating  an  integrated   circuit  implementing  Digital's  		*
**  StrongARM architecture,  regardless  of the  source of such integrated 	*
**	circuit, provided that the  above copyright  notice and this permission *
**  notice appear  in  all copies,  and  that  the  name  of  Digital  		*
**  Equipment Corporation  not  be  used  in advertising or publicity 		*
**  pertaining to distribution of the  document  or  software without 		*
**  specific, written prior permission.							            *
**                                                                          *
**  Digital  Equipment  Corporation   disclaims  all   warranties  and/or   *
**  guarantees  with  regard  to  this  software,  including  all implied   *
**  warranties of fitness for  a  particular purpose and merchantability,   *
**  and makes  no  representations  regarding  the use of, or the results   *
**  of the use of, the software and documentation in terms of correctness,  *
**  accuracy,  reliability,  currentness  or  otherwise;  and you rely on   *
**  the software, documentation and results solely at your own risk.	    *
**                                                                          *
**  StrongARM is a trademark of Digital Equipment Corporation               *
**                                                                          *
*****************************************************************************/


#ifndef __SOCKET_H_
#define __SOCKET_H_

#include "address.h"
#include "udp.h"

#define DEVICE 0 /* The stack requires a device number,to eliminate the need for
					device resolution (Angel needs only 1 device) */
/*
 * Socket Types
 */
#define SOCK_STREAM     1               /* stream socket */
#define SOCK_DGRAM      2               /* datagram socket */
#define SOCK_RAW        3               /* raw-protocol interface */
#define SOCK_RDM        4               /* reliably-delivered message */
#define SOCK_SEQPACKET  5               /* sequenced packet stream */

#define NUM_SUPPORTED_TYPES 1			/* Number of socket types supported */
#define NUM_OF_SOCK_TYPES 5

/*
	The Address Family defines 
*/
#define AF_UNSPEC       0       /* unspecified */
#define AF_INTRA        1       /* O/S-dependent path names */
#define AF_INET         2       /* ARPA Internet address */
#define AF_NS           3       /* Xerox Network Systems protocols */
#define AF_NBS          4       /* NBS protocols */
#define AF_CCITT        5       /* CCITT protocols, X.25 etc */
#define AF_SNA          6       /* IBM SNA */
#define AF_ARP          7       /* Address Resolution Protocol */
#define AF_KDB          8       /* Kernel DeBug Protocol */
#define AF_RARP         9       /* Reverse Address Resolution Protocol */

#define NUM_SUPPORTED_DOMAINS	1

/* Protocol types */ 
/* IP protocol codes  - once again, as per ARM */
#define IP_IP           0       /* Internet Protocol */
#define IP_ICMP         1       /* Internet Control Message Protocol */
#define IP_GGP          3       /* Gateway-to-Gateway Protocol */
#define IP_TCP          6       /* Transaction Control Protocol */
#define IP_UDP          17      /* User Datagram Protocol */
#define IP_SNMP         57     

/* Bit fields of iph_saddr & iph_daddr */

#define INADDR_ANY      ((unsigned long)0)  /* unknown, or Berkeley broadcast address */
#define IP_BROADCAST    ((unsigned long)0xFFFFFFFF) /* The real broadcast address */

/* Error Types */
#define SE_OUTOFMEMORY -1
#define	SE_INVALIDPROTOCOL -2
#define SE_INVALIDFAMILY -3


/*-----------------------------------**
** Defines for status of struct etc  **
**-----------------------------------*/

/*
	Errors
*/
#define	EGENERAL 		-1
#define EWOULDBLOCK 	-2


/* Don't want the sign bit or a = zero */
#define NONZEROINC(a)  'a=(0==(a=(0x7fffff & ++a)))?1:a'

enum {SOCK_UNUSED=0, SOCK_BUSY};


/* Structures */
typedef struct sockaddr {
        unsigned short  sa_family;       
        char    sa_data[14];            
} sockaddr;

typedef struct in_addr {
        unsigned long s_addr;
} in_addr;

typedef struct sockaddr_in
{
	unsigned short 	sin_family; /* Network type, eg AF_INET */	
	unsigned short 	sin_port; 
    struct in_addr 	sin_addr; /* The ip address */
    char   sin_zero[8];
} sockaddr_in;



typedef struct packet_queue {
	struct packet_queue *next;
	int age;
	int device_no;			/* id of physical hw device */
	unsigned char* packet;	/* ptr to start of data for us */
	int size;				/* size of data at packet */
	udp_port source_port;	/* id of port given at creation */
	udp_port dest_port;
	ip_addr source_address;	/* source IP address */
	mac_addr mac;			/* src mac address */
	unsigned char* frame_buffer; /* ptr to start of entire packet buffer */
} packet_queue;


/* Internal tables  */
	
typedef struct inuse_socket_table {
	int 	socket_num;
	udp_port port_num;
	unsigned short type; 	
	unsigned short protocol; 
	union {
	sockaddr sa;
	sockaddr_in	si; 
	} s;
	struct inuse_socket_table *next;
} inuse_socket_table;



/* Function Prototypes */

int     bind(int socket, struct sockaddr_in *localaddr , int addrlen); /* bind socket number to local protocol port */
int     connect(int socket,  struct sockaddr *destaddr, int addrlen); /* bind socket to destination address */
int     getpeername(int socket, struct sockaddr *destaddr, int *addrlen );
int     getsockname(int socket, struct sockaddr *loclaaddr, int *addrlen);
int 	recv(int socket,   void *buf, int len, int flags);
int 	recvfrom(int socket, void *buf, int len, int flags, struct sockaddr *fromaddr, int *addrlen);
int 	send(int socket, const void *msg, int len, int flags);
int 	sendto(int socket, const void *msg, int len, int flags, const struct sockaddr *destaddr, int addrlen);
int     socket(int af, int type, int protocol);
int     socketpair (int af, int type , int protocol, int *sarray);
int 	close(int socket);
void	socket_init(void);
unsigned short 	ntohs(unsigned short netshort); /* Convert network byte order to host order */
unsigned int	ntohl(unsigned int netlong); /* net long to host long */
unsigned short	htons(unsigned short localshort); /* host short to network order */
unsigned int 	htonl(unsigned int locallong); /* host long to network order */
int		nonblocking(int socket);

#endif /* End of __SOCKET_H */

