//
// Copyright (C) 1997 by
// Digital Equipment Corporation, Maynard, Massachusetts.
// All rights reserved.
//
// This software is furnished under a license and may be used and copied
// only  in  accordance  of  the  terms  of  such  license  and with the
// inclusion of the above copyright notice. This software or  any  other
// copies thereof may not be provided or otherwise made available to any
// other person.  No title to and  ownership of the  software is  hereby
// transferred.
//
// The information in this software is  subject to change without notice
// and  should  not  be  construed  as a commitment by Digital Equipment
// Corporation.
//
// Digital assumes no responsibility for the use  or  reliability of its
// software on equipment which is not supplied by Digital.
//
//
// Abstract:    
//
// Author:      
//
// Revision:    
//
// Modifications:
//
//     $History: $
//
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "platform.h"
#ifndef ANGEL
#include "cdef.h"
#endif
#include "fmu.h"
#ifndef ANGEL
#include "pciproto.h"
#endif

/*
 *  Global variables
 */
extern unsigned volatile char *flash_base ;
unsigned int flash_block_size = 0x10000 ;
extern char blocks[NUMBER_OF_BLOCKS] ;

/*
 *  forward declarations.
 */
int check_flash_id(volatile void *flash) ;
unsigned int read_flash_dword(int offset) ;
int write_flash_dword(int offset, unsigned int data) ;
void delete_flash_block(int blockno) ;

static unsigned char read_flash_byte(int offset) ;
static int write_flash_byte(int offset, unsigned char data) ;
static void writeLowByteAddr(unsigned char);

/* ---------- F L A S H --- S U P P O R T --- R O U T I N E S ----------------*/

/*
 *  Check to see if there is some flash at the location specified.
 */
int check_flash_id(volatile void *flash) 
{
    unsigned volatile char *flashPtr = (unsigned volatile char *)flash ;
    unsigned char man, id ;

    /* Interrupts use the firmware, which we can't use while doing things to the flash */
    /* read the manufacturer's id */
    writeLowByteAddr((unsigned char)((U32)flashPtr&3));
    *flashPtr = 0xff ;        
    writeLowByteAddr((unsigned char)((U32)flashPtr&3));
    *flashPtr = 0x90 ;        
    man = *flashPtr ;        

    /* correct manufacturer's id? */
    if (man != 0x89) {
       /* Make sure the flashPtr (if it exists) is back in normal mode so 
          that we can do user I/O via firmware in flashPtr */
       writeLowByteAddr((unsigned char)((U32)flashPtr&3));
       *flashPtr = 0xff;
       return FALSE ;
    } 

    /* read the device id */
    id = *(flashPtr + 1) ;
    /* correct device id? */
    if (id < 0xA1) {
       /* Make sure the flashPtr (if it exists) is back in normal mode so 
          that we can do user I/O via firmware in flashPtr */
       writeLowByteAddr((unsigned char)((U32)flashPtr&3));
       *flashPtr = 0xff;
       return FALSE ; 
    }

    /* whoopee, we found it */

    /* Make sure the flashPtr is back in normal mode so 
       that we can do user I/O via firmware in flashPtr */
    writeLowByteAddr((unsigned char)((U32)flashPtr&3));
    *flashPtr = 0xff;
    return TRUE ;
}

/* relies on flash_base to be correctly set */
unsigned int read_flash_dword(int offset)
{
  int i ;
  unsigned int result ;

  result = 0 ;
  for (i = 0 ; i < 4 ; i++) {
    result |= (read_flash_byte(offset+i) << (i*8)) ;
  }

  return result ;
}

/* relies on flash_base to be correctly set */
static unsigned char read_flash_byte(int offset)
{
#ifndef ANGEL
  //
  // this code will notice that the offset into the rom 
  // needs to swapped...
  //
  if (offset <= 0x1F) {
    offset += 0x20;
  }
  else {
    if ((offset >= 0x20) && (offset < 0x40)) {
      offset -= 0x20;
    }
  }
#endif
  return  *((unsigned char*)(flash_base + offset));

}

/* relies on flash_base to be correctly set */
int write_flash_dword(int offset, unsigned int data)
{
  int i ;
  unsigned char c ;

  for (i = 0 ; i < 4 ; i++) {
    c = data & 0xFF ;
    data = data >> 8 ;
    if (!write_flash_byte(offset++, c) )
      return FALSE ;
  }
  return TRUE ;
}

/* relies on flash_base to be correctly set */
static int write_flash_byte(int offset, unsigned char data)
{
  int status ; 

#ifndef ANGEL
  //
  // this code will notice that the offset into the rom 
  // needs to swapped...
  //
  if (offset <= 0x1F) {
    offset += 0x20;
  }
  else {
    if ((offset >= 0x20) && (offset < 0x40)) {
      offset -= 0x20;
    }
  }
#endif

  /* set the device into write mode */
  writeLowByteAddr((unsigned char)((U32)(flash_base+offset)&3));
  *(flash_base + (int)(offset&0xFFFFFFFC)) = 0x40 ;
  
  /* write the data */
  writeLowByteAddr((unsigned char)((U32)(flash_base+offset)&3));
  *(flash_base + (int)(offset&0xFFFFFFFC)) = data ; 

  /* successful? */
  do {
    writeLowByteAddr((unsigned char)((U32)(flash_base+offset)&3));
    *(flash_base + (int)(offset&0xFFFFFFFC)) = 0x70 ;
    status = *(flash_base + offset) ;
  } while ((status & 0x80) != 0x80) ;
  
  /* Make sure the flash is back in normal mode so 
     that we can do user I/O via firmware in flash */
  writeLowByteAddr((unsigned char)((U32)(flash_base)&3));
  *flash_base = 0xff;
  /* a write error? */
  if ((status & 0x10) == 0x10) {
    status = FALSE ;
  } else 
    status = TRUE ;

  return status ;
}

void delete_flash_block(int blockno) 
{
  unsigned int status ;
  unsigned int offset = (blockno * flash_block_size) ;

  /* write delete block command followed by confirm */
  writeLowByteAddr((unsigned char)((U32)(flash_base+offset)&3));
  *(flash_base + offset) = 0x20 ; 
  writeLowByteAddr((unsigned char)((U32)(flash_base+offset)&3));
  *(flash_base + offset) = 0xd0 ;

  /* check the status */
  do {
    writeLowByteAddr((unsigned char)((U32)(flash_base+offset)&3));
    *(flash_base + offset) = 0x70 ;
    status = *(flash_base + offset);
  } while ((status & 0x80) != 0x80) ;
  if ((status & 0x20) == 0x20) {
    /* Make sure the flash is back in normal mode so 
       that we can do user I/O via firmware in flash */
    writeLowByteAddr((unsigned char)((U32)(flash_base)&3));
    *flash_base = 0xff;
    return ;
  }
  /* Make sure the flash is back in normal mode so 
     that we can do user I/O via firmware in flash */
  
  writeLowByteAddr((unsigned char)((U32)(flash_base)&3));
  *flash_base = 0xff;

  blocks[blockno] = (char) -1 ;
}

static void writeLowByteAddr(unsigned char byteEn)
{
#ifndef ANGEL   
  extern deviceInfo dInfo ;
  volatile unsigned char x ; 
  unsigned int *byteReg = (unsigned int*) (dInfo.bar0Virt + 0x68);

   //
   // this read will sync the inbound queue with the write data
   //
   x = *flash_base;
#else
  unsigned int *byteReg = (unsigned int*) (CSR_BASE + ROM_BYTE_WRITE);
#endif

   //
   // here do the write we are interested in
   //
   *byteReg = byteEn;
}    
