//
// Copyright (C) 1997 by
// Digital Equipment Corporation, Maynard, Massachusetts.
// All rights reserved.
//
// This software is furnished under a license and may be used and copied
// only  in  accordance  of  the  terms  of  such  license  and with the
// inclusion of the above copyright notice. This software or  any  other
// copies thereof may not be provided or otherwise made available to any
// other person.  No title to and  ownership of the  software is  hereby
// transferred.
//
// The information in this software is  subject to change without notice
// and  should  not  be  construed  as a commitment by Digital Equipment
// Corporation.
//
// Digital assumes no responsibility for the use  or  reliability of its
// software on equipment which is not supplied by Digital.
//
//
// Abstract:    
//
// Author:      
//
// Revision:    
//
// Modifications:
//
//     $History: $
//
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "platform.h"
#ifndef ANGEL
#include "cdef.h"
#endif
#include "fmu.h"
#ifndef ANGEL
#include "pciproto.h"
#endif

struct aif_header {
  unsigned int decompress ;
  unsigned int SelfReloc ;
  unsigned int DBGinit ;
  unsigned int EntryPoint ;
  unsigned int Exit ;
  unsigned int ROSize ;
  unsigned int RWSize ;
  unsigned int DebugSize ;
  unsigned int ZeroInitSize ;
  unsigned int ImageDebugType ;
  unsigned int ImageBase ;
} typedef aif_t ;

/* NOP instruction */
#define NOP      0xe1a00000 
/* bitmask to check for BL instruction */
#define BL       0x0a000000

/* Macro to create branch instructions */
#define BINSTR(offset) (0xeb000000+(((offset)-8)>>2))

#define NOBOOTMASK 1

struct image_header {
  int            Type ;
  unsigned char  Number ;
  unsigned char  Sig[3] ;
  int            Map ;
  int            Checksum ;
  int            Length ;
  unsigned char  Name[16]  ;
  unsigned int   Bootflags ; 
  unsigned char  Reserved[24] ;
} typedef image_header_t ;

#define IMAGE_HEADER_SIZE 64 

static struct image_header *header = NULL ;

/*
 *  Global variables
 */
unsigned volatile char *flash_base = NULL ;
extern unsigned int flash_block_size ;

char blocks[NUMBER_OF_BLOCKS] ;

#define BLOCK_IN_USE(n) (blocks[n] != ((char) -1))

#ifndef ANGEL
deviceInfo dInfo ;
#endif

/*
 *  Externals.
 */
extern unsigned int read_flash_dword(int offset) ;
extern int write_flash_dword(int offset, unsigned int data) ;
extern void delete_flash_block(int blockno) ;
extern int check_flash_id(volatile void *flash) ;

/*
 *  forward declarations.
 */
static int write_flash_header(int offset, aif_t *aif, int unique, char *name, int map, int size, int file_checksum);

/* read an AIF header and check it */
static int read_aif_header(FILE *file, aif_t *aif) 
{
  unsigned char data ;
  int i ;

  /* Read the AIF image header */
  for (i = 0 ; i < sizeof(aif_t) ; i++) {
    data = fgetc(file) ;
    if (data == EOF) {
      printf("ERROR: reading file\n") ;
      fclose(file) ;
      return -1 ;
    }
    *((char *)(aif) + i) = data ;
  }
  
  /* reset the file to offset 0 */
  fseek(file, 0, SEEK_SET) ;

  /* return success */
  return 0 ;
}

long fsize(FILE *file)
{
  long offset ;

  fseek(file, 0, SEEK_END) ;
  offset = ftell(file) ;
  fseek(file, 0, SEEK_SET) ;

  return offset ;
}

/* truncate the filename at the first . to generate an image name,
   crude but effective */
static void filename_to_imagename(char *filename)
{
  int i = 0;

  while (filename[i] != '\0') {
    if (filename[i] == '.') {
      filename[i] = '\0' ;
      return ;
    } else
      i++ ;
  }
}

/* program an image into flash */
static int program_flash(int number, 
			  char *filename)
{
  FILE *file ;
  aif_t aif ;
  int i, j, offset ;
  int size ;
  unsigned int data ;
  int checksum;
  int blocks ;

  /* open the file */
  file = fopen(filename, "rb") ;
  if (file == NULL) {
    printf("ERROR: failed to open file\n") ;
    return 0 ;
  }

  /* is there room? */
  size = (int)fsize(file) ;
  blocks = (size + IMAGE_HEADER_SIZE) / flash_block_size ;
  if (blocks > 1) {
    printf("ERROR: image occupies more than one flash block\n") ;
    return 0 ;
  }

  if (((size + IMAGE_HEADER_SIZE) % flash_block_size) != 0) blocks++ ;

  /* read the aif header into memory (to play with) */
  if (read_aif_header(file, &aif) < 0)  {
    fclose(file) ;
    return 0 ; 
  }

  /* tell the world what is going on */
  printf("Writing %s into flash block %d",
	 filename, number) ;

  /* delete the flash blocks that this image will take up */
  printf("\nDeleting blocks ready to program:\n") ;
  delete_flash_block(number) ;

  /* Calculate the file checksum. The checksum is simply the (little
     endian) 32 bit sum of the 32 bit words of the file. 
     Overflow is ignored, and excess bytes in the last word are treated 
     as 0xff (since this is the value of unprogrammed bytes of a flash).
     The checksum of the header will added in in write_flash_header */
  printf("Calculating checksum\n");
  checksum=0;
  fseek(file, 0, SEEK_SET) ;
  for(i=0;i<size;i++) {
      checksum += fgetc(file) << ((i%4)*8);
  }
  for(i=((size-1)%4)+1;i<4;i++) {
      checksum += 0xff << i*8;
  }

  /* convert the filename to an image name */
  filename_to_imagename(filename) ;

  /* build an image header and write it to flash */
  offset = number * flash_block_size ;
  offset = write_flash_header(offset, 
			      &aif, 
			      number, 
			      filename, 
			      1 << number,
			      size, 
			      checksum) ;
  if  (offset < 0) {
    fclose(file) ;
    return 0 ;
  }

  /* write the whole file into flash */
  fseek(file, 0, SEEK_SET) ;
  printf("Writing image file\n") ;
  for (i = 0; i < size; i += 4) {
    data = 0 ;
    for (j = 0 ; j < 4 ; j++) {
      data |= (fgetc(file) << (j * 8)) ;
    }
    write_flash_dword(offset, data) ;
    offset += 4 ;
  }

  /* close the file */
  fclose(file) ;

  return 1 ;
}

/* --------- I M A G E --- S U P P O R T --- R O U T I N E S ----------------*/
static int write_flash_header(int offset, aif_t *aif, int unique, char *name, int map, int size, int file_checksum) 
{
  int count, i;
  unsigned int *data = NULL ;
  int pad = 0 ;
  int checksum;
  int * header_p;
  

  /* tell the world */
  printf("Writing flash image header\n") ;

  /* zero out the header */
  memset((char *)header, 0, sizeof(image_header_t))  ;

  /* work out what sort of image it is */
  if ((aif->EntryPoint & BL) == BL) {
    printf("Image is Executable AIF file\n") ;
    printf("Image will execute from Flash\n") ;
    /* Create a BL to the start of the aif header (The aif header itself is executable) */
    header->Type = BINSTR(0x40)  ; 
  } else {
      printf("Image is non-executable AIF file\n") ;
      pad = IMAGE_HEADER_SIZE - sizeof(image_header_t) ;
      if((aif->ImageBase > FLASH_BASE) && (aif->ImageBase < FLASH_END)) {
	  /* The base address is within the EPROM or FLASH area. This
	   * only makes sense if the image is to be executed from FLASH.
	   */
	  
	  /* Check that the bottom bits of the base address are correct */
	  if((aif->ImageBase & 0x00ffffff) != offset+0xc0) {
	      printf("ERROR: image base address is in ROM but does not match address of flash block\n") ;
	      printf("       Image base address = 0x%x\n",aif->ImageBase);
	      printf("       Expected start of image = 0x??%06x\n",offset+0xc0);
	      return -1;
	  }
	  printf("Image will execute from Flash\n") ;
	  /* Put in a branch to the entry point */
	  header->Type= BINSTR((0xc0 + aif->EntryPoint));
      } else {
	  printf("The bootloader will copy this image to %x before executing it\n",aif->ImageBase);
	      printf("       Eprom is from 0x%x to 0x%x\n", FLASH_BASE, FLASH_END) ;
	  /* The bootloader will relocate the image */
	  header->Type = NOP ;
      }
  }

  header->Number = unique ;

  header->Sig[0] = 0x55 ;
  header->Sig[1] = 0xaa ;
  header->Sig[2] = 0x00 ;

  header->Map = map ;
  header->Bootflags = 0 ;

  /* it has a name */
  for (i = 0 ; i < sizeof(header->Name); i++) header->Name[i] = ' ';
  for (i = 0 ; i < strlen(name) ; i++) header->Name[i] = name[i] ;

  /* it's size */
  header->Length = size + IMAGE_HEADER_SIZE ;

  /* work out the checksum */
  header->Checksum = 0 ;

  checksum=file_checksum;
  header_p = (int *)header;
  for(i=0; i<sizeof(image_header_t); i+=4) {
      checksum += *(header_p++);
  }
  /* Negate the checksum so that the final checksum of the whole image is 0 */
  header->Checksum = -checksum;
  
  /* write it out */
  count = sizeof(image_header_t) ;
  data = (unsigned int *)header ;
  do {
    if (!write_flash_dword(offset, *data++)) {
      printf("ERROR: writing flash header at offset 0x%x (%d)\n",
	     offset, offset) ;
      return -1 ;
    }
    offset += 4 ;
    count  -= 4 ;
  } while (count > 0) ;

  printf("padding is %d bytes\n", pad) ;
  /* pad it out */
  while (pad) {
    if (!write_flash_dword(offset++, 0)) {
      printf("ERROR: writing flash header at offset 0x%x (%d)\n",
	     offset, offset) ;
      return -1 ;
    }
    pad -= 4 ;
  }
     
  /* return successfully */
  return offset ;
}

/* ------------------------ M A I N ---------------------------- */
#define manuBanner "Digital EBSA-285 Flash Manufacturing Utility [v1.1] (DOS)"

#ifdef ANGEL

int main(int argc, char *argv[])
{
  int c ;

  /* try and find the flash */
  printf("Searching for flash device\n") ;
  if (check_flash_id((unsigned char *)FLASH_BASE) == TRUE) {
    flash_base = (unsigned char *)FLASH_BASE ;
  } else {
    printf("Failed to find flash\n") ;
    exit(1) ;
  }
  printf("Flash found at 0x%p (%d blocks of size 0x%x)\n", 
	 flash_base, NUMBER_OF_BLOCKS, flash_block_size) ;

  header = (struct image_header *)malloc(IMAGE_HEADER_SIZE) ;
  if (header == NULL) {
    printf("ERROR: failed to allocate memory\n") ;
    exit(1) ;
  }

  /* program the image(s) into flash */
  for (c = 1; c < argc; c++) 
    if (program_flash( c - 1, argv[c]) != 1) exit(1) ;

  /* return normally */
  exit(0) ;
}
#else
int main(int argc, char *argv[])
{    
  int c;
  biosInfo bInfo;
  devInfo device;
#if defined SCAN_VALID  
  unsigned int x;
#endif  
  unsigned int done=0,problem=0;
  PU32 tPntr;

  /* tell the world just who we are */
  printf(manuBanner);

  //
  // make sure that the system supports PCI bios calls.
  //
  if (checkPciBios(&bInfo)==MSG_FAILURE) {
    printf("\nouch ! PCI BIOS not detected !\n");
    exit(1);
  }

  if (bInfo.hwMech == 0) {
    printf("ouch ! problem with Config Mechanism detection !\n");
    exit(1);
  }

  printf("PCI Config Mechanism: #%d, BIOS version: %d.%d\n",
                                 bInfo.hwMech&3, bInfo.major,bInfo.minor);

  //
  // try and find device
  //
  printf("Searching for Digital DC21285 device\r");

  if (findDevice(0x1011,0x1065,0,&device) == MSG_FAILURE) {
    printf("unable to locate any Digital DC21285 devices\n");
    exit(1);
  }

  dInfo.bus = device.bus;
  dInfo.slot = device.slot;
  dInfo.function = device.function;
  dInfo.bar0Phys = incfgd(0,dInfo.bus,dInfo.slot,dInfo.function,0x10);

  printf("Found Digital DC21285 device on bus:%d, slot:%d\n",dInfo.bus,
          dInfo.slot);
  dInfo.romPhys = 0xB0000000;

  //
  // we need to map 2 areas of memory. The first will allow us to access
  // the 128 memory mapped CSR's for the rogue low byte address feature, 
  // the second will create a 16 MB area that we can map the flash ROM into.
  // Sooooo, this code will scan looking for 16 MB sections for us to use 
  // for ROM space. Chances are that this first location will work fine,
  // but we need to prepare for the worst.
  //
  while (!done) {  

    printf("scanning region 0x%08X\r",dInfo.romPhys);
    if (dpmiMap(dInfo.romPhys,0x1000000,(PU32)&dInfo.romVirt,0) == MSG_FAILURE) {
      printf("ouch ! unable to allocate DPMI memory to map ROM\n");
      exit(1);
    }

    tPntr = (PU32)dInfo.romVirt;

    //
    // since we are flying by the seat of our pants here, we need to make
    // sure that no other device is present. BIOS does not have a nice
    // call that would allocate non-existant memory space for us to map
    // our ROM to. So scan through the region expecting master aborts 
    // throughout.
    //
#if defined SCAN_VALID
    for (x=0;x<0x1000000/4;x++) {

      //
      // we didn't get a master abort
      //
      if (tPntr[x] != 0xFFFFFFFF) {

        //
        // set issue flag
        //
        problem = 1;

        //
        // bump our region to the next aligned 16 MB region
        //
        dInfo.romPhys += 0x1000000;

        //
        // free up the last allocated region
        //
        dpmiFree(dInfo.romVirt,0x1000000,0);
      }
    }
#endif

    //
    // if there were no problems, then we have a good region
    //
    if (problem == 0) 
      done = 1;
    else 

      //
      // else reset the issue flag
      //
      problem = 0;

    //
    // make sure that we don't fall off of the end of memory 
    // this is where the system ROM is.
    //
    if (dInfo.romPhys == 0xF0000000) {
      printf("ouch ! unable to locate available memory for ROM\n");
      exit(1);
    }   
  } 
  
  printf("Expansion ROM BAR mapped to physical memory at 0x%08X\n",dInfo.romPhys);

  //
  // write the Expansion ROM register with our region location
  //
  outcfgd(0,dInfo.bus,dInfo.slot,dInfo.function,0x30,dInfo.romPhys|1);    

  //
  // map the CSR's into memory space
  //
  if (dpmiMap(dInfo.bar0Phys,0x80,(PU32)&dInfo.bar0Virt,0) == MSG_FAILURE) {
    printf("ouch ! unable to allocate DPMI memory to map 21285 CSR's\n");
    exit(1);
  }
  /* try and find the flash */
  printf("Searching for flash device\n") ;

#ifdef REAL_DEVICE  
  if (check_flash_id((unsigned char *)dInfo.romVirt) == TRUE) {
    flash_base = (unsigned char *)dInfo.romVirt;
  }
  else {
    printf("Failed to find flash\n") ;
    exit(1) ;
  }
  printf("Flash found at 0x%p (%d blocks of size 0x%x)\n", 
        dInfo.romPhys, NUMBER_OF_BLOCKS, flash_block_size) ;

#else
  flash_base = (unsigned char *)dInfo.romVirt;

  printf("Flash found at 0x%p (%d blocks of size 0x%x)\n", 
        dInfo.romPhys, NUMBER_OF_BLOCKS, flash_block_size) ;
#endif

  header = (struct image_header *)malloc(IMAGE_HEADER_SIZE) ;
  if (header == NULL) {
    printf("ERROR: failed to allocate memory\n") ;
    exit(1) ;
  }

/* program the image(s) into flash */
for (c = 1; c < argc; c++) 
  if (program_flash( c - 1, argv[c]) != 1) exit(1) ;

//
// free the buffers
//
dpmiFree(dInfo.romVirt,0x1000000,0);
dpmiFree(dInfo.bar0Virt,0x80,0);

/* return normally */
exit(0) ;
}
#endif
