;/*****************************************************************************
; 
;       Copyright  1997 Digital Equipment Corporation,
;                       Maynard, Massachusetts.
; 
;                        All Rights Reserved
; 
;Permission to use, copy, modify, and distribute this software and its 
;documentation for any purpose and without fee is hereby granted, provided  
;that the copyright notice and this permission notice appear in all copies  
;of software and supporting documentation, and that the name of Digital not  
;be used in advertising or publicity pertaining to distribution of the software 
;without specific, written prior permission. Digital grants this permission 
;provided that you prominently mark, as not part of the original, any 
;modifications made to this software or documentation.
; 
;Digital Equipment Corporation disclaims all warranties and/or guarantees  
;with regard to this software, including all implied warranties of fitness for 
;a particular purpose and merchantability, and makes no representations 
;regarding the use of, or the results of the use of, the software and 
;documentation in terms of correctness, accuracy, reliability, currentness or
;otherwise; and you rely on the software, documentation and results solely at 
;your own risk. 
;
;******************************************************************************
; 
;  PowerOn Selftest code for the SA-110  based EBSA285 evaluation board.
; 
;	$Id: io.s,v 1.8 1997/09/17 09:40:00 rusling Exp $
;
;******************************************************************************/
; EBSA285 Manufacturing & Power On Self Tests: I/O drivers
;
; History:	
;
		KEEP
		; assembler support
		EXPORT ResetCOMPort
		EXPORT PrintString
		EXPORT PrintHexWord
		EXPORT PutByte
		EXPORT GetByte
		EXPORT SetLEDs
		EXPORT Pause

		; C support
		EXPORT ResetAMBA
		EXPORT PutString
		EXPORT PutHex
		EXPORT GetHex
		EXPORT cpuIdRead
		EXPORT cpuControlRead
		EXPORT	__rt_stkovf_split_small
		EXPORT	__rt_stkovf_split_big
	
		; include files
		INCLUDE platform.s
		INCLUDE	superio_h.s
			
		AREA	SelfTests_IO,CODE,READONLY

 IF EB285_USE_UARTS <> 0
PORT	EQU	COM1
 ENDIF
; ------------------------------------------------------------------------------
; Reset functions
; 
ResetCOMPort
;; Set the default line speed
 IF EB285_USE_UARTS <> 0
		ldr	r2,=PORT
		ldr	r1, =0			; disable interrupt 
		strb	r1, [r2, #IntEnable]
		ldr	r1, =DLABMsk		; Set divisor access
		strb	r1, [r2, #LineCntl]
		ldr	r1, =Baud9600low	; Set 9600 low divisor
		strb	r1, [r2, #Dllsb]
		ldr	r1, =Baud9600high	; set 9600 high divisor
		strb	r1, [r2, #Dlmsb]
		ldr	r1, =WordLen8		; Clear divisor access
		strb	r1, [r2, #LineCntl]
		ldr	r1, =0			; 0 disables the FIFOs
		strb	r1, [r2, #FIFOcntl]
		ldrb	r1, [r2, #LineStatus]	; read status will clear errors
		ldr	r0, =3
		strb	r0, [r2, #ModemCntl]	; disable modem control
		strb	r0, [r2, #IntEnable]	; Hack - Enable Interrupts

 ELSE
		ldr	r2,=HOST_COMPORT
		ldr	r1, =0			; disable interrupts
		strb	r1, [r2, #AMBA_UARTCON]

		ldr	r1, =0			; set the baud rate to 9600
		strb	r1, [r2, #AMBA_M_UBRLCR]
		ldr	r1, =0x50
		strb	r1, [r2, #AMBA_L_UBRLCR]

		ldr	r1, =AMBA_H_UBRLCR_DSS_8; 8 bits, 1 stop, no parity, no fifo
		strb	r1, [r2, #AMBA_H_UBRLCR]

		ldr	r1, =AMBA_UARTCON_UE	; enable the uart
		strb	r1, [r2, #AMBA_UARTCON]		
 ENDIF

		mov     pc, lr			; return 
; ------------------------------------------------------------------------------
;
; Low level input functions
; 
; Return the next byte from the serial lines
 IF EB285_USE_UARTS <> 0
GetByte		LDR	r0, =PORT
GetByteLoop	LDRB	r1, [r0,#LineStatus]	; Wait for a character, note
		TST	r1, #LineDRMsk		; that reading this register
		BEQ	GetByteLoop		; also clears any errors 			
		LDRB	r0, [r0, #Rx]
 ELSE
GetByte		LDR	r0, =HOST_COMPORT
GetByteLoop	LDRB	r1, [r0,#AMBA_UARTFLG]	; Wait for a character, note
		ANDS	r1, r1, #AMBA_UARTFLG_RFE
		BNE	GetByteLoop
		LDRB	r0, [r0, #AMBA_UARTDR]
 ENDIF
                MOV     pc, lr
; ------------------------------------------------------------------------------
;
;	Low level print functions
; 
 IF EB285_USE_UARTS <> 0
PutByte		LDR	r1, =PORT
PutByteLoop	LDRB	r2, [r1, #LineStatus]
		TST	r2, #LineTEMTMsk	; Wait until ready to queue character
		BEQ	PutByteLoop

		STRB	r0, [r1, #Tx]
 ELSE
PutByte		LDR	r1, =HOST_COMPORT
PutByteLoop	LDRB	r2, [r1, #AMBA_UARTFLG]
		TST	r2, #AMBA_UARTFLG_TFF	; Wait until ready to queue character
		BNE	PutByteLoop
		STRB	r0, [r1, #AMBA_UARTDR]
 ENDIF
		MOV	pc, lr
; ------------------------------------------------------------------------------
;
; Higher level printing functions
;
PrintString     MOV     r4, lr
                MOV     r5, r0
PutAChar        LDRB    r0, [r5], #1
                TST     r0, #&ff
                MOVEQ   pc, r4
                BL      PutByte
                B       PutAChar

; Print a four byte word in hex, preceded by ' 0x'

PrintHexWord    MOV     r4, lr
                MOV     r5, r0
                MOV     r0, #&20 ; ' '
                BL      PutByte
                MOV     r0, #&30 ; '0'
                BL      PutByte
                MOV     r0, #&78 ; 'x'
                BL      PutByte
                B       PrintWord2

	; Print a four byte word in hex

PrintWord       MOV     r4, lr
                MOV     r5, r0
PrintWord2      MOV     r3, #28
01              MOV     r0, r5, LSR r3
                ANDS    r0, r0, #&f
                BNE     %F02 ; the first non-zero
                SUBS    r3, r3, #4
                BNE     %B01 ; all but the last digit
		MOV	r0, r5 ; Get the only digit
02              CMP     r0, #10
                ADDLO   r0, r0, #&30 ; 0 - 9
                ADDHS   r0, r0, #&61 - 10 ; a - f
                BL      PutByte
                SUBS    r3, r3, #4
                MOV     r0, r5, LSR r3
                AND     r0, r0, #&f
                BPL     %B02
                MOV     pc, r4

; ------------------------------------------------------------------------------
; Switch the LEDs off and on
SetLEDs
		EOR	r0, r0, #ALL_LEDS
		LDR	r1, =LED_BANK
		STRB	r0, [r1]
		MOV	pc, lr

; ------------------------------------------------------------------------------
; Pause
Pause
		LDR	r0, =0x10000
WaitLoop
		SUBS	r0, r0, #1
		BNE	WaitLoop
		MOV	pc, lr
	
; ------------------------------------------------------------------------------
; C support routines
;
; STacK OVerFlow handling routine - minimum required by ARM Linker without
; defining no stack checking.
; void __rt_stkovf_split_big(void);
|__rt_stkovf_split_big|
		MOV	pc, lr			; just return



; STacK OVerFlow handling routine - minimum required by ARM Linker without
; defining no stack checking.
; void __rt_stkovf_split_small(void);
|__rt_stkovf_split_small|
		MOV	pc, lr			; just return


; ------------------------------------------------------------------------------
; Jacket routines, callable from C.
ResetAMBA
		STMFD	sp!, {r0-r10, lr}	; save working registers
		BL	ResetCOMPort
		LDMFD	sp!, {r0-r10, pc}	; restore registers & return
		
PutString
		STMFD	sp!, {r0-r10, lr}	; save working registers
		BL	PrintString		; Print the passed string 
		LDMFD	sp!, {r0-r10, pc}	; restore registers & return
		
PutHex
		STMFD	sp!, {r0-r10, lr}	; save working registers
		BL	PrintHexWord		; Print the passed number
		LDMFD	sp!, {r0-r10, pc}	; restore registers & return

GetHex		STMFD	sp!, {r1-r2, lr}	; save working registers
		BL	GetByte			; returns char in r0
		LDMFD	sp!, {r1-r2, pc}	; restore registers & return
			
cpuControlRead
;
; read the control register into r0
;
	        MRC     p15, 0, r0, c1, c0
		MOV	pc, lr
        
cpuIdRead
;
; read the ID into r0
;
	        MRC     p15, 0, r0, c0, c0
		MOV	pc, lr
	
		END

