# s2h.awk
#
# 12-Jan-1996 NAC Created
#
# Take a .s file containing ;-delimited comments and EQUates
# and convert it to #defines and c-style comments for a c .h file
#
# 11-Nov-1996 MJQ
#
# Added better comment handling & support for 'C' #defines, #ifdefs etc.
#
# 22-Nov-1996 MJQ
#
# Now with INCLUDE .s -> #include .h & IF, ELSE, ENDIF support
#
# 25-Apr-1997 MJQ
#
# EQU's can now have spaces. IFs now work as well as IF :DEF:s - didn't before

BEGIN {
  FALSE = 0
  TRUE = -1
  print("")
  print("/* DO NOT EDIT!! - this file automatically generated")
  print(" *                 from .s file by awk -f s2h.awk")
  print(" */")
  comment = 0
  include = 0
  equate = 0
  c_comment = 0
  c_define = 0
  end_c_comment = 0
}

{
  comment = index ($0, ";")
  include = index ($0, "INCLUDE")
  equate = index ($0, "EQU")
  c_comment = index ($0, "/*")
  end_c_comment = index ($0, "*/")
  if (($2 ~ /EQU/) && ((comment==0) || (comment > equate )))
  {
     # this line contains an equate, and the equate is not commented out.
     # The equate line may have a comment at the end of it.

     # Let's get the EQU to define out of the way..
     printf("#define %-30s  %s",$1, substr($0, index($0, $3), length($0)))
     if (comment != 0)
     {
	# there is a comment at the end of the line
	# Watch out for 'C'-style comments inside comments
	if (c_comment == 0)
	   printf(" /* %s", substr($0, comment+1, length($0)))
	else
	   printf(" %s", substr($0, c_comment, length($0)))

        if (end_c_comment == 0)
	   printf(" */");
     }

     printf("\n");
  }
  else if ((include != 0) && ((comment==0) || (comment > include )))
  {
     # this line contains an include, and the include is not commented out.

     # Let's get the INCLUDE to include out of the way..
     printf("#include \"%sh\" ", substr($2, 1, index($2, ".")))
     printf("\n");
     # IGNORE COMMENTS (if any)
  }
  else
  {
     if (comment == 0)
     {
	# Can't have ANY comments with IF, ELSE, ENDIF
	if (($1 ~ /ENDIF/) || ($1 ~ /\]/))
           printf("#endif\n")
	else if (($1 ~ /IF/) || ($1 ~ /\[/)) 
	{
	   if ($2 ~ /:DEF:/)
              printf("#ifdef %s\n",$3)
	   else
              printf("#if %s\n", substr($0, index($0, $2), length($0)))
	}
	else if (($1 ~ /ELSE/) || ($1 ~ /\|/))
           printf("#else\n")

        # probably a blank line. Just echo it..
        else if (index ($0, "END") == 0)
           print($0)
        else
           printf("/* %s */\n",$0)
     }
     else
     {
        # the line is a comment line
        if (comment > index ($0, $1))
	{
	   # ..but there's some stuff before the semi-colon.

	   # Watch out for END & a comment!
           end = index ($0, "END")
	   if ((end != 0) && (comment > end))
	   {
               printf("/* ")
	       # Don't print out another start of comment
	       if (c_comment == 0)
	          c_comment = comment + 1
	   }
	   # echo upto start of comment
	   printf("%s ", substr($0, 1, comment-1));
	}
	# Do a special check for ;; as a single comment
	end_comment = index($0, ";;")
	if (end_comment == 0)
	   end_comment = comment 
	else
	   end_comment = end_comment + 1

	# Look for 'C' #defines etc. _inside_ comments
	c_define = index ($0, "#if")
	if (c_define == 0)
	   c_define = index ($0, "#endif")
	if (c_define == 0)
	   c_define = index ($0, "#else")
	if (c_define == 0)
	   c_define = index ($0, "#define")
	if (c_define == 0)
	   c_define = index ($0, "#include")
	if (c_define != 0)
	{
           # It's really a 'C' #define - ignore before & print line
	   printf("%s", substr($0, c_define, length($0)))
	}
	else
	{
	   # Whatever is left is just a comment.

	   # Watch out for 'C'-style comments inside comments
	   if (c_comment == 0)
	      printf("/* %s", substr($0, end_comment+1, length($0)))
	   else
	      printf("%s", substr($0, c_comment, length($0)))

           if (end_c_comment == 0)
	      printf(" */");
	}

     printf("\n");
     }
  }
}

